module register_wrapper
#(
	parameter INSIG_WIDTH  = 0,
	parameter OUTSIG_WIDTH = 0
)
(
	clk,
	reset_n,

	insig_in,
	outsig_out
);

localparam COMPLETION_INFO_WIDTH = 64;

input	wire							clk;
input	wire							reset_n;

input	[  ( INSIG_WIDTH - 1 ) : 0 ]	insig_in;
output	[ ( OUTSIG_WIDTH - 1 ) : 0 ]	outsig_out;

reg		[  ( INSIG_WIDTH - 1 ) : 0 ]	reg_in;
reg		[ ( OUTSIG_WIDTH - 1 ) : 0 ]	reg_out;

wire	[  ( INSIG_WIDTH - 1 ) : 0 ]	in_vect;
wire	[ ( OUTSIG_WIDTH - 1 ) : 0 ]	out_vect;

reg										my_reset_n;

//
// instance input signal wires
//
wire  clk_sig;
wire  reset_sig;
wire [15:0] asi_devcsrsnk_data_sig;
wire [15:0] asi_linkcsrsnk_data_sig;
wire [12:0] asi_busdevsnk_data_sig;
wire [COMPLETION_INFO_WIDTH-1:0] asi_cplinfosnk_data_sig;
wire  asi_cplinfosnk_valid_sig;
wire [63:0] asi_rddatasnk_data_sig;
wire  asi_rddatasnk_valid_sig;
wire  aso_tlpsrc_ready_sig;

//
// instance output signal wires
//
wire  asi_cplinfosnk_ready_sig;
wire  asi_rddatasnk_ready_sig;
wire [63:0] aso_tlpsrc_data_sig;
wire  aso_tlpsrc_valid_sig;
wire  aso_tlpsrc_startofpacket_sig;
wire  aso_tlpsrc_endofpacket_sig;

//
// assign clock and reset drivers
//
assign clk_sig = clk;
//assign FIXME_reset_n_sig = my_reset_n;	//  my_reset_n for negative asserted reset
assign reset_sig = ~my_reset_n;		// ~my_reset_n for positive asserted reset

//
// map input and output registers to input and output vectors
//
assign in_vect = reg_in;
assign outsig_out = reg_out;

//
// map input and output vectors to instance inputs and outputs
//
assign {
	asi_devcsrsnk_data_sig,
	asi_linkcsrsnk_data_sig,
	asi_busdevsnk_data_sig,
	asi_cplinfosnk_data_sig,
	asi_cplinfosnk_valid_sig,
	asi_rddatasnk_data_sig,
	asi_rddatasnk_valid_sig,
	aso_tlpsrc_ready_sig
} = in_vect;

assign out_vect = {
	asi_cplinfosnk_ready_sig,
	asi_rddatasnk_ready_sig,
	aso_tlpsrc_data_sig,
	aso_tlpsrc_valid_sig,
	aso_tlpsrc_startofpacket_sig,
	aso_tlpsrc_endofpacket_sig
};

always @ ( posedge clk ) begin
	
		my_reset_n <= reset_n;
end

always @ ( posedge clk or negedge my_reset_n ) begin
	
	if( ~my_reset_n ) begin
	
		reg_in <= 0;
		reg_out <= 0;
		
	end else begin
	
		reg_in <= insig_in;
		reg_out <= out_vect;

	end
end

//
// instanctiate test instance here
//
completion_generator_64b completion_generator_64b_inst
(
	.clk(clk_sig) ,	// input  clk_sig
	.reset(reset_sig) ,	// input  reset_sig
	.asi_devcsrsnk_data(asi_devcsrsnk_data_sig) ,	// input [15:0] asi_devcsrsnk_data_sig
	.asi_linkcsrsnk_data(asi_linkcsrsnk_data_sig) ,	// input [15:0] asi_linkcsrsnk_data_sig
	.asi_busdevsnk_data(asi_busdevsnk_data_sig) ,	// input [12:0] asi_busdevsnk_data_sig
	.asi_cplinfosnk_data(asi_cplinfosnk_data_sig) ,	// input [COMPLETION_INFO_WIDTH-1:0] asi_cplinfosnk_data_sig
	.asi_cplinfosnk_ready(asi_cplinfosnk_ready_sig) ,	// output  asi_cplinfosnk_ready_sig
	.asi_cplinfosnk_valid(asi_cplinfosnk_valid_sig) ,	// input  asi_cplinfosnk_valid_sig
	.asi_rddatasnk_data(asi_rddatasnk_data_sig) ,	// input [63:0] asi_rddatasnk_data_sig
	.asi_rddatasnk_ready(asi_rddatasnk_ready_sig) ,	// output  asi_rddatasnk_ready_sig
	.asi_rddatasnk_valid(asi_rddatasnk_valid_sig) ,	// input  asi_rddatasnk_valid_sig
	.aso_tlpsrc_data(aso_tlpsrc_data_sig) ,	// output [63:0] aso_tlpsrc_data_sig
	.aso_tlpsrc_ready(aso_tlpsrc_ready_sig) ,	// input  aso_tlpsrc_ready_sig
	.aso_tlpsrc_valid(aso_tlpsrc_valid_sig) ,	// output  aso_tlpsrc_valid_sig
	.aso_tlpsrc_startofpacket(aso_tlpsrc_startofpacket_sig) ,	// output  aso_tlpsrc_startofpacket_sig
	.aso_tlpsrc_endofpacket(aso_tlpsrc_endofpacket_sig) 	// output  aso_tlpsrc_endofpacket_sig
);

defparam completion_generator_64b_inst.COMPLETION_INFO_WIDTH = COMPLETION_INFO_WIDTH;

endmodule
