//----------------------------------------------------------------------------------------------------
//-- (c)2010 Altera Corporation. All rights reserved.
//--
//-- Altera products are protected under numerous U.S. and foreign patents,
//-- maskwork rights, copyrights and other intellectual property laws.
//--
//-- This reference design file, and your use thereof, is subject to and governed
//-- by the terms and conditions of the applicable Altera Reference Design License
//-- Agreement (either as signed by you or found at www.altera.com).  By using
//-- this reference design file, you indicate your acceptance of such terms and
//-- conditions between you and Altera Corporation.  In the event that you do not
//-- agree with such terms and conditions, you may not use the reference design
//-- file and please promptly destroy any copies you have made.
//--
//-- This reference design file is being provided on an as-is basis and as an
//-- accommodation and therefore all warranties, representations or guarantees of
//-- any kind (whether express, implied or statutory) including, without
//-- limitation, warranties of merchantability, non-infringement, or fitness for
//-- a particular purpose, are specifically disclaimed.  By making this reference
//-- design file available, Altera expressly does not recommend, suggest or
//-- require that this reference design file be used in combination with any
//-- other product not provided by Altera.
//----------------------------------------------------------------------------------------------------

//	
//	This module receives Avalon master commands from the tlp2master_64b component and translates those
//	into Avalon MM Master transactions.  It then passes readdata returned to read transactions back out
//	the read data Avalon ST source interface to the completion_generator peripheral.
//	

module bar_master_64b
#(
	parameter AVALON_MASTER_ADDRESS_WIDTH = 16
)
(
	// Clock Interface
	input  wire				clk,
	input  wire				reset,
	
	// Avalon Master Command Input Interface - Avalon ST
	input  wire	[ ( AVALON_MASTER_ADDRESS_WIDTH + 83 - 1 ) : 0 ]	asi_cmdsnk_data,
	output wire				asi_cmdsnk_ready,
	input  wire				asi_cmdsnk_valid,

	// Avalon Master Read Data Output Interface - Avalon ST
	output wire	[ 63 : 0 ]	aso_rddatasrc_data,
	input  wire				aso_rddatasrc_ready,
	output wire				aso_rddatasrc_valid,

	// Master Interface - Avalon MM
	output wire	[ ( AVALON_MASTER_ADDRESS_WIDTH - 1 ) : 0 ]	avm_m0_address,
	output wire	[  7 : 0 ]	avm_m0_byteenable,
	output wire	[  9 : 0 ]	avm_m0_burstcount,
	output wire				avm_m0_read,
	output wire				avm_m0_write,
	output wire	[ 63 : 0 ]	avm_m0_writedata,
	input  wire				avm_m0_waitrequest,
	input  wire	[ 63 : 0 ]	avm_m0_readdata,
	input  wire				avm_m0_readdatavalid
);

//
// Master Command Input Elements
//
wire [ ( AVALON_MASTER_ADDRESS_WIDTH - 1 ) : 0 ] avm_address;
wire [  7 : 0 ]	avm_byteenable;
wire [  9 : 0 ]	avm_burstcount;
wire			avm_write;
wire [ 63 : 0 ] avm_writedata;

assign { avm_address, avm_byteenable, avm_burstcount, avm_write, avm_writedata } = asi_cmdsnk_data;

assign avm_m0_address = avm_address;
assign avm_m0_byteenable = avm_byteenable;
assign avm_m0_burstcount = avm_burstcount;
assign avm_m0_writedata = avm_writedata;

assign avm_m0_read = asi_cmdsnk_valid & ~avm_write;
assign avm_m0_write = asi_cmdsnk_valid & avm_write;

assign asi_cmdsnk_ready = ( avm_m0_read | avm_m0_write ) & ~avm_m0_waitrequest;

//
// readdata output Pipeline
//
bar_master_64b_1stage_pipeline #( .PAYLOAD_WIDTH( 64 ) ) readdata_outpipe (
    .clk            ( clk ),
    .reset_n        ( ~reset ),
    .in_ready       ( ),
    .in_valid       ( avm_m0_readdatavalid ), 
    .in_payload     ( avm_m0_readdata ),
    .out_ready      ( aso_rddatasrc_ready ), 
    .out_valid      ( aso_rddatasrc_valid ), 
    .out_payload    ( aso_rddatasrc_data )
);

endmodule

//  --------------------------------------------------------------------------------
// | single buffered pipeline stage
//  --------------------------------------------------------------------------------
module bar_master_64b_1stage_pipeline  
#( parameter PAYLOAD_WIDTH = 8 )
 ( input                          clk,
   input                          reset_n, 
   output reg                     in_ready,
   input                          in_valid,   
   input      [PAYLOAD_WIDTH-1:0] in_payload,
   input                          out_ready,   
   output reg                     out_valid,
   output reg [PAYLOAD_WIDTH-1:0] out_payload      
 );
      
   always @* begin
     in_ready <= out_ready || ~out_valid;
   end
   
   always @ (negedge reset_n, posedge clk) begin
      if (!reset_n) begin
         out_valid <= 0;
         out_payload <= 0;
      end else begin
         if (in_valid) begin
           out_valid <= 1;
         end else if (out_ready) begin
           out_valid <= 0;
         end
         
         if(in_valid && in_ready) begin
            out_payload <= in_payload;
         end
      end
   end

endmodule
