//----------------------------------------------------------------------------------------------------
//-- (c)2010 Altera Corporation. All rights reserved.
//--
//-- Altera products are protected under numerous U.S. and foreign patents,
//-- maskwork rights, copyrights and other intellectual property laws.
//--
//-- This reference design file, and your use thereof, is subject to and governed
//-- by the terms and conditions of the applicable Altera Reference Design License
//-- Agreement (either as signed by you or found at www.altera.com).  By using
//-- this reference design file, you indicate your acceptance of such terms and
//-- conditions between you and Altera Corporation.  In the event that you do not
//-- agree with such terms and conditions, you may not use the reference design
//-- file and please promptly destroy any copies you have made.
//--
//-- This reference design file is being provided on an as-is basis and as an
//-- accommodation and therefore all warranties, representations or guarantees of
//-- any kind (whether express, implied or statutory) including, without
//-- limitation, warranties of merchantability, non-infringement, or fitness for
//-- a particular purpose, are specifically disclaimed.  By making this reference
//-- design file available, Altera expressly does not recommend, suggest or
//-- require that this reference design file be used in combination with any
//-- other product not provided by Altera.
//----------------------------------------------------------------------------------------------------

//	
//	This module strips the data payload out of the completion TLP packet to forward on to the 
//	multichannel FIFO.
//	

module completion_data_extration_64b
#(
	parameter CHANNEL_WIDTH = 2
)
(
	// Clock Interface
	input  wire				clk,
	input  wire				reset,
	
	// Input Interface - Avalon ST
	input  wire	[ 63 : 0 ]	asi_snk_data,
	input  wire	[ ( CHANNEL_WIDTH - 1 ) : 0 ]	asi_snk_channel,
	output wire				asi_snk_ready,
	input  wire				asi_snk_valid,
	input  wire				asi_snk_startofpacket,
	input  wire				asi_snk_endofpacket,

	// Output Interface - Avalon ST
	output wire	[ 63 : 0 ]	aso_src_data,
	output wire	[ ( CHANNEL_WIDTH - 1 ) : 0 ]	aso_src_channel,
	input  wire				aso_src_ready,
	output wire				aso_src_valid,
	output wire				aso_src_startofpacket,
	output wire				aso_src_endofpacket
);

localparam CHANNEL_COUNT = 2 ** CHANNEL_WIDTH;

integer i;

reg	 [ ( CHANNEL_COUNT - 1 ) : 0 ] sop_states;
reg	 [ ( CHANNEL_COUNT - 1 ) : 0 ] sop_states_on_mask;
reg	 [ ( CHANNEL_COUNT - 1 ) : 0 ] sop_states_off_mask;
reg				this_sop_state;

wire 			in_ready;
wire 			in_valid;
wire 			in_startofpacket;
wire 			in_endofpacket;
wire [ 63 : 0 ]	pipe_payload_data;

wire			final_data_packet;
reg				reg_final_data_packet;

reg  			sop_plus1;
reg  			sop_plus2;
reg				completion_status_error;
reg  [  9 : 0 ] reg_length_field;
reg  [ 11 : 0 ] reg_bytecount_field;
reg  [  6 : 0 ]	reg_loweraddress_field;
reg  			reg_bcm_field;

//
// Completion TLP header word 0
//
wire [  0 : 0 ] tlp_res3_field;
wire [  1 : 0 ] tlp_fmt_field;
wire [  4 : 0 ] tlp_type_field;
wire [  0 : 0 ] tlp_res2_field;
wire [  2 : 0 ] tlp_tc_field;
wire [  3 : 0 ] tlp_res1_field;
wire [  0 : 0 ] tlp_td_field;
wire [  0 : 0 ] tlp_ep_field;
wire [  1 : 0 ] tlp_attr_field;
wire [  1 : 0 ] tlp_res0_field;
wire [  9 : 0 ] tlp_length_field;

//
// Completion TLP header word 1
//
wire [ 15 : 0 ] tlp_completerid_field;
wire [  2 : 0 ] tlp_cplstatus_field;
wire 			tlp_bcm_field;
wire [ 11 : 0 ] tlp_bytecount_field;

//
// Completion TLP header word 2
//
wire [ 15 : 0 ] tlp_reqid_field;
wire [  7 : 0 ] tlp_tag_field;
wire 			tlp_res4_field;
wire [  6 : 0 ]	tlp_loweraddress_field;

assign {
			tlp_completerid_field,
			tlp_cplstatus_field,
			tlp_bcm_field,
			tlp_bytecount_field,
			
			tlp_res3_field,
			tlp_fmt_field,
			tlp_type_field,
			tlp_res2_field,
			tlp_tc_field,
			tlp_res1_field,
			tlp_td_field,
			tlp_ep_field,
			tlp_attr_field,
			tlp_res0_field,
			tlp_length_field
							} = asi_snk_data;
							
assign {
		tlp_reqid_field,
		tlp_tag_field,
		tlp_res4_field,
		tlp_loweraddress_field
							} = asi_snk_data[ 31 : 0 ];

always @ ( posedge clk or posedge reset ) begin

	if( reset ) begin
	
		sop_plus1 <= 0;
		sop_plus2 <= 0;
		reg_final_data_packet <= 0;
		reg_length_field <= 0;
		reg_bytecount_field <= 0;
		reg_bcm_field <= 0;
		completion_status_error <= 0;
		reg_loweraddress_field <= 0;
		
	end else begin
		
		if( asi_snk_valid & asi_snk_ready ) begin
	
			sop_plus1 <= asi_snk_startofpacket;
			sop_plus2 <= sop_plus1;
			
			if( asi_snk_startofpacket ) begin

				reg_length_field <= tlp_length_field;
				reg_bytecount_field <= tlp_bytecount_field;
				reg_bcm_field <= tlp_bcm_field;
				
				completion_status_error <= |tlp_cplstatus_field;
			end

			if( sop_plus1 ) begin

				reg_loweraddress_field <= tlp_loweraddress_field;
				reg_final_data_packet <= final_data_packet;
			end
		end
	end
end

always @ (*) begin
	this_sop_state = 0;
	sop_states_on_mask = 0;
	sop_states_off_mask = { CHANNEL_COUNT { 1'b1 } };
	for ( i = 0 ; i < ( CHANNEL_COUNT - 1 ) ; i = i + 1 )
		if ( asi_snk_channel == i ) begin
			this_sop_state = sop_states[i];
			sop_states_on_mask[i] = 1;
			sop_states_off_mask[i] = 0;
		end else begin
			sop_states_on_mask[i] = 0;
			sop_states_off_mask[i] = 1;
		end
end

always @ ( posedge clk or posedge reset ) begin
	if( reset ) begin
	
		sop_states <= 0;
		
	end else begin
		if( in_ready & in_valid & in_startofpacket & ~in_endofpacket ) begin
		
			sop_states <= sop_states | sop_states_on_mask;
			
		end else if( in_ready & in_valid & in_endofpacket ) begin

			sop_states <= sop_states & sop_states_off_mask;
		end
	end
end

assign final_data_packet =	( reg_bcm_field == 0 ) ?
								( { reg_length_field, 2'b00 } >= ( reg_bytecount_field + tlp_loweraddress_field[ 1 : 0 ] ) ) ? 
									( 1'b1 ) : 
									( 1'b0 ) :
								( 1'b0 );

assign in_startofpacket =	( ( sop_plus1 & tlp_loweraddress_field[2] ) || ( sop_plus2 & ~reg_loweraddress_field[2] ) ) ?
								( ~this_sop_state ) :
								( 1'b0 );

assign in_endofpacket = ( sop_plus1 & asi_snk_endofpacket & final_data_packet ) |
						( asi_snk_endofpacket & reg_final_data_packet );

assign asi_snk_ready =	( asi_snk_startofpacket ) ?						// sop?
							( 1'b1 ) :										// sop - take this word immediately, discard data
							( sop_plus1 & ~tlp_loweraddress_field[2] ) ?	// !sop - sop_plus1 and aligned?
								( 1'b1 ) :										// sop_plus1 and aligned - take this word immediately, discard data
								( in_ready );									// sop_plus1 and unaligned or all others - synchronize into outpipe
							

assign in_valid =	( ~asi_snk_startofpacket & ~sop_plus1 ) ?		// !sop & !sop+1?
						( asi_snk_valid ) :								// !sop & !sop+1? - synchronize with sink input
						( sop_plus1 & tlp_loweraddress_field[2] ) ?		// sop | sop+1 - sop+1 and unaligned?
							( asi_snk_valid ) :								// sop+1 and unaligned - synchronize with sink input
							( 1'b0 );										// all others - don't pass data to outpipe
//
// readdata output Pipeline
//
completion_data_extration_64b_1stage_pipeline #( .PAYLOAD_WIDTH( 66 + CHANNEL_WIDTH ) ) outpipe (
    .clk            ( clk ),
    .reset_n        ( ~reset ),
    .in_ready       ( in_ready ),
    .in_valid       ( in_valid ), 
    .in_payload     ( { asi_snk_data, asi_snk_channel, in_startofpacket, in_endofpacket } ),
    .out_ready      ( aso_src_ready ), 
    .out_valid      ( aso_src_valid ), 
    .out_payload    ( { aso_src_data, aso_src_channel, aso_src_startofpacket, aso_src_endofpacket } )
);

endmodule

//  --------------------------------------------------------------------------------
// | single buffered pipeline stage
//  --------------------------------------------------------------------------------
module completion_data_extration_64b_1stage_pipeline  
#( parameter PAYLOAD_WIDTH = 8 )
 ( input                          clk,
   input                          reset_n, 
   output reg                     in_ready,
   input                          in_valid,   
   input      [PAYLOAD_WIDTH-1:0] in_payload,
   input                          out_ready,   
   output reg                     out_valid,
   output reg [PAYLOAD_WIDTH-1:0] out_payload      
 );
      
   always @* begin
     in_ready <= out_ready || ~out_valid;
   end
   
   always @ (negedge reset_n, posedge clk) begin
      if (!reset_n) begin
         out_valid <= 0;
         out_payload <= 0;
      end else begin
         if (in_valid) begin
           out_valid <= 1;
         end else if (out_ready) begin
           out_valid <= 0;
         end
         
         if(in_valid && in_ready) begin
            out_payload <= in_payload;
         end
      end
   end

endmodule
