//----------------------------------------------------------------------------------------------------
//-- (c)2010 Altera Corporation. All rights reserved.
//--
//-- Altera products are protected under numerous U.S. and foreign patents,
//-- maskwork rights, copyrights and other intellectual property laws.
//--
//-- This reference design file, and your use thereof, is subject to and governed
//-- by the terms and conditions of the applicable Altera Reference Design License
//-- Agreement (either as signed by you or found at www.altera.com).  By using
//-- this reference design file, you indicate your acceptance of such terms and
//-- conditions between you and Altera Corporation.  In the event that you do not
//-- agree with such terms and conditions, you may not use the reference design
//-- file and please promptly destroy any copies you have made.
//--
//-- This reference design file is being provided on an ?as-is? basis and as an
//-- accommodation and therefore all warranties, representations or guarantees of
//-- any kind (whether express, implied or statutory) including, without
//-- limitation, warranties of merchantability, non-infringement, or fitness for
//-- a particular purpose, are specifically disclaimed.  By making this reference
//-- design file available, Altera expressly does not recommend, suggest or
//-- require that this reference design file be used in combination with any
//-- other product not provided by Altera.
//----------------------------------------------------------------------------------------------------

//  
//  This module receives read completion information from the tlp2master peripheral along with readdata
//  from the bar_master peripheral, and it combines this into completion TLP packets to forward back out
//  to the PCIe core.
//  

module completion_generator_64b
#(
    parameter COMPLETION_INFO_WIDTH = 64
)
(
    // Clock Interface
    input  wire             clk,
    input  wire             reset,
    
    // DEVCSR Input Interface - Avalon ST
    input  wire [ 15 : 0 ]  asi_devcsrsnk_data,
    
    // LINKCSR Input Interface - Avalon ST
    input  wire [ 15 : 0 ]  asi_linkcsrsnk_data,
    
    // BUSDEV Input Interface - Avalon ST
    input  wire [ 12 : 0 ]  asi_busdevsnk_data,
    
    // Avalon Master Read Completion Info Input Interface - Avalon ST
    input  wire [ ( COMPLETION_INFO_WIDTH - 1 ) : 0 ]   asi_cplinfosnk_data,
    output wire             asi_cplinfosnk_ready,
    input  wire             asi_cplinfosnk_valid,

    // Avalon Master Read Data Input Interface - Avalon ST
    input  wire [ 63 : 0 ]  asi_rddatasnk_data,
    output wire             asi_rddatasnk_ready,
    input  wire             asi_rddatasnk_valid,
    
    // TLP Output Interface - PCIe ST
    output wire [ 63 : 0 ]  aso_tlpsrc_data,
    input  wire             aso_tlpsrc_ready,
    output wire             aso_tlpsrc_valid,
    output wire             aso_tlpsrc_startofpacket,
    output wire             aso_tlpsrc_endofpacket
    
);

//
// Completion State Values
//
localparam STATE_HDR0_HDR1  = 0;
localparam STATE_HDR2       = 1;
localparam STATE_DATA       = 2;

reg  [  2 : 0 ] state;

//
// Completion Output Pipeline
//
wire            completion_outpipe_ready;
wire            completion_outpipe_valid;
wire [ 63 : 0 ] completion_data;
wire            completion_sop;
wire            completion_eop;

//
// Completion Info Elements
//
wire [  1 : 0 ] cpl_info_attr;
wire [  2 : 0 ] cpl_info_tc;
wire [  7 : 0 ] cpl_info_tag;
wire [ 15 : 0 ] cpl_info_reqid;
wire [  3 : 0 ] cpl_info_lbe;
wire [  3 : 0 ] cpl_info_fbe;
wire [  9 : 0 ] cpl_info_length;
wire [  6 : 0 ] cpl_info_loaddr;
wire [  9 : 0 ] cpl_info_burstcount;

wire [ 31 : 0 ] cpl_header_0;
wire [ 31 : 0 ] cpl_header_1;
wire [ 31 : 0 ] cpl_header_2;

wire [  2 : 0 ] cpl_fbe_sum;
wire [  2 : 0 ] cpl_lbe_sum;

wire [ 12 : 0 ] cpl_total_read_bytes;

wire [  6 : 0 ] cpl_info_loaddr_plus_1;
wire [  6 : 0 ] cpl_info_loaddr_plus_2;
wire [  6 : 0 ] cpl_info_loaddr_plus_3;

wire [  6 : 0 ] first_loweraddress;

wire [  6 : 0 ] cpl_loweraddress;

wire            loweraddress_is_rcb_aligned;

wire [ 11 : 0 ] cpl_bytecount;

wire [ 12 : 0 ] max_bytes_this_completion;

reg  [ 12 : 0 ] reg_bytes_this_completion;
wire [ 12 : 0 ] actual_bytes_this_completion;

wire [ 12 : 0 ] actual_bytes_this_completion_plus3_plus;

wire [ 9 : 0 ] actual_dw_this_completion;

wire [  9 : 0 ] cpl_length;

wire [  3 : 0 ] first_st_data_byte_count;

wire [  3 : 0 ] last_st_data_byte_count;

reg  [ 12 : 0 ] accumulated_read_bytes;
reg  [  9 : 0 ] accumulated_st_data_words;
wire [ 12 : 0 ] next_accumulated_read_bytes;
wire [  9 : 0 ] next_accumulated_st_data_words;

wire            cpl_eop_next_st_data;
wire            cpl_completed_next_st_data;

//
// Misc Status Inputs
//
wire rcb_field;
wire [ 7 : 0 ] rcb_size;

wire [  2 : 0 ] max_payload_size_field;
wire [ 12 : 0 ] max_payload_size;

wire [  7 : 0 ] bus_number_field;

wire [  4 : 0 ] dev_number_field;
//
// Completion Info Elements
//
assign  { 
            cpl_info_attr,
            cpl_info_tc,
            cpl_info_tag,
            cpl_info_reqid,
            cpl_info_lbe,
            cpl_info_fbe,
            cpl_info_length,
            cpl_info_loaddr,
            cpl_info_burstcount
        } = asi_cplinfosnk_data;
        
assign cpl_header_0 = { 1'b0, 2'b10, 5'b01010, 1'b0, cpl_info_tc, 4'b0000, 1'b0, 1'b0, cpl_info_attr, 2'b00, cpl_length };
assign cpl_header_1 = { bus_number_field, dev_number_field, 3'b000, 3'b000, 1'b0, cpl_bytecount };
assign cpl_header_2 = { cpl_info_reqid, cpl_info_tag, 1'b0, cpl_loweraddress };

assign cpl_fbe_sum = cpl_info_fbe[0] + cpl_info_fbe[1] + cpl_info_fbe[2] + cpl_info_fbe[3];
assign cpl_lbe_sum = cpl_info_lbe[0] + cpl_info_lbe[1] + cpl_info_lbe[2] + cpl_info_lbe[3];

assign cpl_total_read_bytes =   ( cpl_info_length == 1 ) ? ( cpl_fbe_sum ) :
                                ( cpl_info_length == 2 ) ? ( cpl_fbe_sum + cpl_lbe_sum ) :
                                ( cpl_info_length == 0 ) ? ( { 11'h400, 2'b00 } - ( 3'h4 - cpl_fbe_sum ) - ( 3'h4 - cpl_lbe_sum ) ) :
                                ( { cpl_info_length, 2'b00 } - ( 3'h4 - cpl_fbe_sum ) - ( 3'h4 - cpl_lbe_sum ) );
                                
assign cpl_info_loaddr_plus_1 = cpl_info_loaddr + 2'd1;
assign cpl_info_loaddr_plus_2 = cpl_info_loaddr + 2'd2;
assign cpl_info_loaddr_plus_3 = cpl_info_loaddr + 2'd3;

assign first_loweraddress = ( cpl_info_fbe[ 3 : 1 ] == 3'b000 ) ? ( cpl_info_loaddr ) :             // case 000x
                            ( cpl_info_fbe[ 1 : 0 ] == 2'b10 ) ? ( cpl_info_loaddr_plus_1 ) :       // case xx10
                            ( cpl_info_fbe[ 2 : 0 ] == 3'b100 ) ? ( cpl_info_loaddr_plus_2 ) :      // case x100
                            ( cpl_info_fbe[ 3 : 0 ] == 4'b1000 ) ? ( cpl_info_loaddr_plus_3 ) :     // case 1000
                            ( cpl_info_loaddr );
                            
assign cpl_loweraddress = first_loweraddress + accumulated_read_bytes;

assign loweraddress_is_rcb_aligned =    ( rcb_field == 1'b0 ) ?
                                        ( cpl_loweraddress[ 5 : 0 ] == 5'd0 ) ? ( 1'b1 ) : ( 1'b0 ) :
                                        ( cpl_loweraddress[ 6 : 0 ] == 6'd0 ) ? ( 1'b1 ) : ( 1'b0 );

assign cpl_bytecount = cpl_total_read_bytes - accumulated_read_bytes;

assign max_bytes_this_completion =  ( loweraddress_is_rcb_aligned == 1'b0 ) ?
                                        ( rcb_field == 1'b0 ) ?
                                            ( rcb_size - cpl_loweraddress[ 5 : 0 ] ) :
                                            ( rcb_size - cpl_loweraddress[ 6 : 0 ] ) :
                                        ( max_payload_size );

// This version of the component will always fragment the completion at the first RCB boundary, due to Fmax pressure.
// The piped version of this component is optimized to not fragment for completions < MPS.
assign actual_bytes_this_completion =   ( cpl_bytecount > max_bytes_this_completion ) ?
                                            ( max_bytes_this_completion ) :
                                            ( cpl_bytecount );

assign actual_bytes_this_completion_plus3_plus = actual_bytes_this_completion + 2'd3 + cpl_loweraddress[ 1 : 0 ];

assign actual_dw_this_completion = actual_bytes_this_completion_plus3_plus[ 12 : 2 ];

assign cpl_length = actual_dw_this_completion;

assign first_st_data_byte_count =   ( cpl_info_length == 1 ) ? ( cpl_fbe_sum ) :
                                    ( cpl_info_length == 2 ) ?
                                    ( cpl_info_loaddr[2] == 1'b0 ) ? ( cpl_fbe_sum + cpl_lbe_sum ) : ( cpl_fbe_sum ) :
                                    ( cpl_info_loaddr[2] == 1'b0 ) ? ( cpl_fbe_sum + 4 ) : ( cpl_fbe_sum );

assign last_st_data_byte_count =    ( cpl_info_length == 1 ) ? ( 4'd0 ) :
                                    ( cpl_info_length == 2 ) ?
                                    ( cpl_info_loaddr[2] == 1'b0 ) ? ( 4'd0 ) : ( cpl_lbe_sum ) :
                                    ( cpl_info_length[0] == 1'b0 ) ?
                                    ( cpl_info_loaddr[2] == 1'b0 ) ? ( cpl_lbe_sum + 4 ) : ( cpl_lbe_sum ) :
                                    ( cpl_info_loaddr[2] == 1'b0 ) ? ( cpl_lbe_sum ) : ( cpl_lbe_sum + 4 );

assign next_accumulated_read_bytes =    ( accumulated_read_bytes == 0 ) ? ( accumulated_read_bytes + first_st_data_byte_count ) :
                                        ( next_accumulated_st_data_words == cpl_info_burstcount ) ? ( accumulated_read_bytes + last_st_data_byte_count ) :
                                        ( accumulated_read_bytes + 8 );
                                        
assign next_accumulated_st_data_words = accumulated_st_data_words + 1;

assign cpl_eop_next_st_data = ( next_accumulated_read_bytes >= reg_bytes_this_completion ) ? ( 1'b1 ) : ( 1'b0 );
assign cpl_completed_next_st_data = ( next_accumulated_st_data_words >= cpl_info_burstcount ) ? ( 1'b1 ) : ( 1'b0 );

//
// Misc Status Inputs
//
assign rcb_field = asi_linkcsrsnk_data[3];
assign rcb_size = ( rcb_field == 1'b0 ) ? ( 8'd64 ) : ( 8'd128 );

assign max_payload_size_field = asi_devcsrsnk_data[  7 : 5 ];
assign max_payload_size =   ( max_payload_size_field == 1 ) ? ( 13'd256 ) :
                            ( max_payload_size_field == 2 ) ? ( 13'd512 ) :
                            ( max_payload_size_field == 3 ) ? ( 13'd1024 ) :
                            ( max_payload_size_field == 4 ) ? ( 13'd2048 ) :
                            ( max_payload_size_field == 5 ) ? ( 13'd4096 ) :
                            ( 13'd128 );
                                                        
assign bus_number_field = asi_busdevsnk_data[ 12 : 5 ];

assign dev_number_field = asi_busdevsnk_data[  4 : 0 ];

//
// input interface control
//
assign asi_cplinfosnk_ready = cpl_completed_next_st_data & asi_rddatasnk_ready;

assign asi_rddatasnk_ready =    ( ( state == STATE_HDR2 ) && ( completion_outpipe_valid ) && ( cpl_loweraddress[2] == 1'b1 ) ) ? ( 1'b1 ) :
                                ( ( state == STATE_DATA ) && ( completion_outpipe_valid ) ) ? ( 1'b1 ) : ( 1'b0 );

//
// Completion Packet Synchronization
//

assign completion_outpipe_valid =   ( ( state == STATE_HDR0_HDR1 ) && ( asi_cplinfosnk_valid ) ) ? ( 1'b1 ) :
                                    ( ( state == STATE_HDR2 ) && ( asi_cplinfosnk_valid ) && ( cpl_loweraddress[2] == 1'b0 ) ) ? ( 1'b1 ) :
                                    ( ( state == STATE_HDR2 ) && ( asi_cplinfosnk_valid ) && ( cpl_loweraddress[2] == 1'b1 ) && ( asi_rddatasnk_valid ) ) ? ( 1'b1 ) :
                                    ( ( asi_cplinfosnk_valid ) && ( asi_rddatasnk_valid ) ) ? ( 1'b1 ) : ( 1'b0 );

assign completion_data =    ( state == STATE_HDR0_HDR1 ) ? ( { cpl_header_1, cpl_header_0 } ) :
                            ( state == STATE_HDR2 ) ? ( { asi_rddatasnk_data[ 63 : 32 ], cpl_header_2 } ) :
                            ( asi_rddatasnk_data );

assign completion_sop = ( state == STATE_HDR0_HDR1 ) ? ( 1'b1 ) : ( 1'b0 );

assign completion_eop = ( ( state == STATE_HDR2 ) && ( cpl_eop_next_st_data ) && ( cpl_loweraddress[2] == 1'b1 ) ) ? ( 1'b1 ) :
                        ( ( state == STATE_DATA ) && ( cpl_eop_next_st_data ) ) ? ( 1'b1 ) :
                        ( 1'b0 );

always @ ( posedge clk or posedge reset ) begin
    if( reset ) begin

        state <= STATE_HDR0_HDR1;
        accumulated_read_bytes <= 0;
        accumulated_st_data_words <= 0;
        reg_bytes_this_completion <= 0;
        
    end else begin

        if( completion_outpipe_ready & completion_outpipe_valid ) begin
            case( state )
            STATE_HDR0_HDR1: begin
                
                reg_bytes_this_completion <= accumulated_read_bytes + actual_bytes_this_completion;
                
                state <= STATE_HDR2;
            end
            STATE_HDR2: state <= ( completion_eop == 1'b1 ) ? ( STATE_HDR0_HDR1 ) : ( STATE_DATA );
            STATE_DATA: state <= ( completion_eop == 1'b1 ) ? ( STATE_HDR0_HDR1 ) : ( STATE_DATA );
            default: state <= state; 
            endcase
        end
        
        if( asi_rddatasnk_ready & asi_rddatasnk_valid ) begin

            accumulated_read_bytes <= ( cpl_completed_next_st_data == 1'b0 ) ? ( next_accumulated_read_bytes ) : ( 0 );
            accumulated_st_data_words <= ( cpl_completed_next_st_data == 1'b0 ) ? ( next_accumulated_st_data_words ) : ( 0 );
        end
    end
end

//
// command output Pipeline
//
completion_generator_64b_1stage_pipeline #( .PAYLOAD_WIDTH( 66 ) ) completion_outpipe (
    .clk            (clk),
    .reset_n        (~reset),
    .in_ready       (completion_outpipe_ready),
    .in_valid       (completion_outpipe_valid), 
    .in_payload     ( { completion_data, completion_sop, completion_eop } ),
    .out_ready      (aso_tlpsrc_ready), 
    .out_valid      (aso_tlpsrc_valid), 
    .out_payload    ( { aso_tlpsrc_data, aso_tlpsrc_startofpacket, aso_tlpsrc_endofpacket } )
);

endmodule

//  --------------------------------------------------------------------------------
// | single buffered pipeline stage
//  --------------------------------------------------------------------------------
module completion_generator_64b_1stage_pipeline  
#( parameter PAYLOAD_WIDTH = 8 )
 ( input                          clk,
   input                          reset_n, 
   output reg                     in_ready,
   input                          in_valid,   
   input      [PAYLOAD_WIDTH-1:0] in_payload,
   input                          out_ready,   
   output reg                     out_valid,
   output reg [PAYLOAD_WIDTH-1:0] out_payload      
 );
      
   always @* begin
     in_ready <= out_ready || ~out_valid;
   end
   
   always @ (negedge reset_n, posedge clk) begin
      if (!reset_n) begin
         out_valid <= 0;
         out_payload <= 0;
      end else begin
         if (in_valid) begin
           out_valid <= 1;
         end else if (out_ready) begin
           out_valid <= 0;
         end
         
         if(in_valid && in_ready) begin
            out_payload <= in_payload;
         end
      end
   end

endmodule
