//----------------------------------------------------------------------------------------------------
//-- (c)2010 Altera Corporation. All rights reserved.
//--
//-- Altera products are protected under numerous U.S. and foreign patents,
//-- maskwork rights, copyrights and other intellectual property laws.
//--
//-- This reference design file, and your use thereof, is subject to and governed
//-- by the terms and conditions of the applicable Altera Reference Design License
//-- Agreement (either as signed by you or found at www.altera.com).  By using
//-- this reference design file, you indicate your acceptance of such terms and
//-- conditions between you and Altera Corporation.  In the event that you do not
//-- agree with such terms and conditions, you may not use the reference design
//-- file and please promptly destroy any copies you have made.
//--
//-- This reference design file is being provided on an as-is basis and as an
//-- accommodation and therefore all warranties, representations or guarantees of
//-- any kind (whether express, implied or statutory) including, without
//-- limitation, warranties of merchantability, non-infringement, or fitness for
//-- a particular purpose, are specifically disclaimed.  By making this reference
//-- design file available, Altera expressly does not recommend, suggest or
//-- require that this reference design file be used in combination with any
//-- other product not provided by Altera.
//----------------------------------------------------------------------------------------------------

//	
//	This module simply pipelines the stream and reports ahead to the full_packet_wait_gate peripheral
//	each clock cycle that an endofpacket moves out of this pipeline, so it may manage the full packet
//	credit counter that it maintains.
//	
	
module endofpacket_credit_reporter_64b
(
	// Clock Interface
	input  wire				clk,
	input  wire				reset,
	
	// Input Interface - Avalon ST
	input  wire	[ 63 : 0 ]	asi_snk_data,
	output wire				asi_snk_ready,
	input  wire				asi_snk_valid,
	input  wire				asi_snk_startofpacket,
	input  wire				asi_snk_endofpacket,

	// Output Interface - Avalon ST
	output wire	[ 63 : 0 ]	aso_src_data,
	input  wire				aso_src_ready,
	output wire				aso_src_valid,
	output wire				aso_src_startofpacket,
	output wire				aso_src_endofpacket,
	
	// Credit Increment Report Interface - Avalon ST
	output wire				aso_eopcreditincr_data
);

reg	endofpacket_credit_incr;

always @ ( posedge clk or posedge reset ) begin

	if( reset ) begin
	
		endofpacket_credit_incr <= 0;
		
	end else begin
		
		endofpacket_credit_incr <= aso_src_ready & aso_src_valid & aso_src_endofpacket;
	end
end

assign aso_eopcreditincr_data = endofpacket_credit_incr;

//
// readdata output Pipeline
//
endofpacket_credit_reporter_64b_1stage_pipeline #( .PAYLOAD_WIDTH( 66 ) ) outpipe (
    .clk            ( clk ),
    .reset_n        ( ~reset ),
    .in_ready       ( asi_snk_ready ),
    .in_valid       ( asi_snk_valid ), 
    .in_payload     ( { asi_snk_data, asi_snk_startofpacket, asi_snk_endofpacket } ),
    .out_ready      ( aso_src_ready ), 
    .out_valid      ( aso_src_valid ), 
    .out_payload    ( { aso_src_data, aso_src_startofpacket, aso_src_endofpacket } )
);

endmodule

//  --------------------------------------------------------------------------------
// | single buffered pipeline stage
//  --------------------------------------------------------------------------------
module endofpacket_credit_reporter_64b_1stage_pipeline  
#( parameter PAYLOAD_WIDTH = 8 )
 ( input                          clk,
   input                          reset_n, 
   output reg                     in_ready,
   input                          in_valid,   
   input      [PAYLOAD_WIDTH-1:0] in_payload,
   input                          out_ready,   
   output reg                     out_valid,
   output reg [PAYLOAD_WIDTH-1:0] out_payload      
 );
      
   always @* begin
     in_ready <= out_ready || ~out_valid;
   end
   
   always @ (negedge reset_n, posedge clk) begin
      if (!reset_n) begin
         out_valid <= 0;
         out_payload <= 0;
      end else begin
         if (in_valid) begin
           out_valid <= 1;
         end else if (out_ready) begin
           out_valid <= 0;
         end
         
         if(in_valid && in_ready) begin
            out_payload <= in_payload;
         end
      end
   end

endmodule
