//----------------------------------------------------------------------------------------------------
//-- (c)2010 Altera Corporation. All rights reserved.
//--
//-- Altera products are protected under numerous U.S. and foreign patents,
//-- maskwork rights, copyrights and other intellectual property laws.
//--
//-- This reference design file, and your use thereof, is subject to and governed
//-- by the terms and conditions of the applicable Altera Reference Design License
//-- Agreement (either as signed by you or found at www.altera.com).  By using
//-- this reference design file, you indicate your acceptance of such terms and
//-- conditions between you and Altera Corporation.  In the event that you do not
//-- agree with such terms and conditions, you may not use the reference design
//-- file and please promptly destroy any copies you have made.
//--
//-- This reference design file is being provided on an as-is basis and as an
//-- accommodation and therefore all warranties, representations or guarantees of
//-- any kind (whether express, implied or statutory) including, without
//-- limitation, warranties of merchantability, non-infringement, or fitness for
//-- a particular purpose, are specifically disclaimed.  By making this reference
//-- design file available, Altera expressly does not recommend, suggest or
//-- require that this reference design file be used in combination with any
//-- other product not provided by Altera.
//----------------------------------------------------------------------------------------------------

//	
//	This module provides a simple Avalon Slave interface to the LMI port of the PCIe HIP core.  You must
//	perform 32-bit writes to this interface.  Sub word writes will not be passed along to the LMI port.
//	

module lmi_slave
(
	// Clock Interface
	input  wire				clk,
	input  wire				reset,
	
	// Avalon Slave Interface - Avalon MM
	input  wire	[  9 : 0 ]	avs_lmi_slave_address,
	input  wire	[  3 : 0 ]	avs_lmi_slave_byteenable,
	input  wire				avs_lmi_slave_read,
	input  wire				avs_lmi_slave_write,
	output wire				avs_lmi_slave_waitrequest,
	input  wire	[ 31 : 0 ]	avs_lmi_slave_writedata,
	output wire	[ 31 : 0 ]	avs_lmi_slave_readdata,
	
	// LMI Interface
	output wire 			coe_lmi_rden,
	output wire 			coe_lmi_wren,
	input  wire 			coe_lmi_ack,
	output wire [ 11 : 0 ]	coe_lmi_addr,
	input  wire [ 31 : 0 ]	coe_lmi_dout,
	output wire [ 31 : 0 ]	coe_lmi_din
	
);

assign coe_lmi_rden	= avs_lmi_slave_read;

assign coe_lmi_wren	=	( avs_lmi_slave_byteenable == 4'hF ) ? 
							( avs_lmi_slave_write ) : 
							( 1'b0 );

assign coe_lmi_addr	= { avs_lmi_slave_address, 2'b00 };

assign coe_lmi_din	= avs_lmi_slave_writedata;

assign avs_lmi_slave_waitrequest	=	( coe_lmi_rden & ~coe_lmi_ack ) ?
											( 1'b1 ) :
											( avs_lmi_slave_byteenable == 4'hF ) ?
												( coe_lmi_wren & ~coe_lmi_ack ) :
												( ~avs_lmi_slave_write );

assign avs_lmi_slave_readdata		= coe_lmi_dout;

endmodule
