#ifndef __MSI_INTERFACE_REGS_H__
#define __MSI_INTERFACE_REGS_H__

#include <io.h>

//	Register maps of Avalon Slave CSR
//
//	'h00: [ 31 : 0 ] { 7'h00, mask_capable, addr_64_capable, mult_mess_enable, mult_mess_capable, msi_enable, 13'h00, tc_reg }
//			RW : [  2 : 0 ]	tc_reg	// traffic class for MSI packet
//			RO : msi_enable			// MSI enable status bit from HIP core
//			RO : [ 2 : 0 ]	mult_mess_capable	// Multiple Message Capable status from HIP core
//			RO : [ 2 : 0 ]	mult_mess_enable	// Multiple Message Allocated status from HIP core
//			RO : addr_64_capable		// 64-bit address capable status from HIP core
//			RO : mask_capable			// MSI Maskable status from HIP core
//	'h01: RW : [ 31 : 0 ] irq_enable_reg	// each bit in this register enables the corresponding IRQ input
//	'h02: RO : [ 31 : 0 ] sync_inr_irqin_irq	// this is the live state of each IRQ input
//	'h03: RO : [ 31 : 0 ] msi_sent_reg		// this register indicates that an MSI message has been sent for the corresponding IRQ
//								// these bits are set when the MSI is sent and they are cleared when the IRQ is deactivated at its source

#define MSI_INTERFACE_CNTL_REG					0
#define IORD_MSI_INTERFACE_CNTL_REG(base)				IORD_32DIRECT( base, MSI_INTERFACE_CNTL_REG )
#define IOWR_MSI_INTERFACE_CNTL_REG(base, data)			IOWR_32DIRECT( base, MSI_INTERFACE_CNTL_REG, data )

#define MSI_INTERFACE_CNTL_TC_MASK					( 0x07 )
#define MSI_INTERFACE_CNTL_TC_OFST					( 0 )

#define MSI_INTERFACE_CNTL_MSI_EN_MASK				( 0x01 )
#define MSI_INTERFACE_CNTL_MSI_EN_OFST				( 16 )

#define MSI_INTERFACE_CNTL_MULT_MESS_CAP_MASK			( 0x07 )
#define MSI_INTERFACE_CNTL_MULT_MESS_CAP_OFST			( 17 )

#define MSI_INTERFACE_CNTL_MULT_MESS_EN_MASK			( 0x07 )
#define MSI_INTERFACE_CNTL_MULT_MESS_EN_OFST			( 20 )

#define MSI_INTERFACE_CNTL_ADDR_64_CAP_MASK			( 0x01 )
#define MSI_INTERFACE_CNTL_ADDR_64_CAP_OFST			( 23 )

#define MSI_INTERFACE_CNTL_MASK_CAP_MASK				( 0x01 )
#define MSI_INTERFACE_CNTL_MASK_64_CAP_OFST			( 24 )

#define MSI_INTERFACE_IRQ_EN_REG					1
#define IORD_MSI_INTERFACE_IRQ_EN_REG(base)			IORD_32DIRECT( base, MSI_INTERFACE_IRQ_EN_REG )
#define IOWR_MSI_INTERFACE_IRQ_EN_REG(base, data)		IOWR_32DIRECT( base, MSI_INTERFACE_IRQ_EN_REG, data )

#define MSI_INTERFACE_IRQ_EN_REG_MASK				( 0xFFFFFFFF )
#define MSI_INTERFACE_IRQ_EN_REG_OFST				( 0 )

#define MSI_INTERFACE_IRQ_PENDING_REG				2
#define IORD_MSI_INTERFACE_IRQ_PENDING_REG(base)		IORD_32DIRECT( base, MSI_INTERFACE_IRQ_PENDING_REG )
#define IOWR_MSI_INTERFACE_IRQ_PENDING_REG(base, data)	IOWR_32DIRECT( base, MSI_INTERFACE_IRQ_PENDING_REG, data )

#define MSI_INTERFACE_IRQ_PENDING_REG_MASK			( 0xFFFFFFFF )
#define MSI_INTERFACE_IRQ_PENDING_REG_OFST			( 0 )

#define MSI_INTERFACE_MSI_SENT_REG					3
#define IORD_MSI_INTERFACE_MSI_SENT_REG(base)			IORD_32DIRECT( base, MSI_INTERFACE_MSI_SENT_REG )
#define IOWR_MSI_INTERFACE_MSI_SENT_REG(base, data)		IOWR_32DIRECT( base, MSI_INTERFACE_MSI_SENT_REG, data )

#define MSI_INTERFACE_MSI_SENT_REG_MASK				( 0xFFFFFFFF )
#define MSI_INTERFACE_MSI_SENT_REG_OFST				( 0 )

#endif /* __MSI_INTERFACE_REGS_H__ */
