//----------------------------------------------------------------------------------------------------
//-- (c)2010 Altera Corporation. All rights reserved.
//--
//-- Altera products are protected under numerous U.S. and foreign patents,
//-- maskwork rights, copyrights and other intellectual property laws.
//--
//-- This reference design file, and your use thereof, is subject to and governed
//-- by the terms and conditions of the applicable Altera Reference Design License
//-- Agreement (either as signed by you or found at www.altera.com).  By using
//-- this reference design file, you indicate your acceptance of such terms and
//-- conditions between you and Altera Corporation.  In the event that you do not
//-- agree with such terms and conditions, you may not use the reference design
//-- file and please promptly destroy any copies you have made.
//--
//-- This reference design file is being provided on an as-is basis and as an
//-- accommodation and therefore all warranties, representations or guarantees of
//-- any kind (whether express, implied or statutory) including, without
//-- limitation, warranties of merchantability, non-infringement, or fitness for
//-- a particular purpose, are specifically disclaimed.  By making this reference
//-- design file available, Altera expressly does not recommend, suggest or
//-- require that this reference design file be used in combination with any
//-- other product not provided by Altera.
//----------------------------------------------------------------------------------------------------

//	
//	This module terminates traffic without any concern.
//	

module null_tlp_snk_64b
(

	// Clock Interface
	input  wire				clk,
	input  wire				reset,
	
	// TLP Input Interface - PCIe ST
	input  wire	[ 63 : 0 ]	asi_tlpsnk_data,
	output wire				asi_tlpsnk_ready,
	input  wire				asi_tlpsnk_valid,
	input  wire				asi_tlpsnk_startofpacket,
	input  wire				asi_tlpsnk_endofpacket
);

//
// Things we need to keep out of the TLP header
//
reg  [  1 : 0 ] request_fmt;
reg  [  2 : 0 ] request_tc;
reg  [  1 : 0 ] request_attr;
reg  [  9 : 0 ] request_length;
reg  [ 15 : 0 ] request_reqid;
reg  [  7 : 0 ] request_tag;
reg  [  3 : 0 ] request_lbe;
reg  [  3 : 0 ] request_fbe;

//
// General packet information
//
reg				sop_plus_one;
reg				sop_plus_two;

//
// Things we need to compute
//
reg  [ 63 : 0 ] reg_master_address;

//
// TLP header word 0
//
wire [  0 : 0 ] tlp_res3_field;
wire [  1 : 0 ] tlp_fmt_field;
wire [  4 : 0 ] tlp_type_field;
wire [  0 : 0 ] tlp_res2_field;
wire [  2 : 0 ] tlp_tc_field;
wire [  3 : 0 ] tlp_res1_field;
wire [  0 : 0 ] tlp_td_field;
wire [  0 : 0 ] tlp_ep_field;
wire [  1 : 0 ] tlp_attr_field;
wire [  1 : 0 ] tlp_res0_field;
wire [  9 : 0 ] tlp_length_field;

//
// TLP header word 1
//
wire [ 15 : 0 ] tlp_reqid_field;
wire [  7 : 0 ] tlp_tag_field;
wire [  3 : 0 ] tlp_lbe_field;
wire [  3 : 0 ] tlp_fbe_field;

//
// TLP header word 2 and 3
//
wire [ 61 : 0 ] tlp_address_field;
wire [  1 : 0 ] tlp_res4_field;

assign {
			tlp_reqid_field,
			tlp_tag_field,
			tlp_lbe_field,
			tlp_fbe_field,
			tlp_res3_field,
			tlp_fmt_field,
			tlp_type_field,
			tlp_res2_field,
			tlp_tc_field,
			tlp_res1_field,
			tlp_td_field,
			tlp_ep_field,
			tlp_attr_field,
			tlp_res0_field,
			tlp_length_field
							} = asi_tlpsnk_data;

assign {
			tlp_address_field,
			tlp_res4_field
							} = { asi_tlpsnk_data[ 31 : 0 ], asi_tlpsnk_data[ 63 : 32 ] };

//
// TLP header capture
//

assign asi_tlpsnk_ready = 1;

always @ ( posedge clk or posedge reset ) begin
	if( reset ) begin

		request_fmt				<= 0;
		request_tc				<= 0;
		request_attr			<= 0;
		request_length			<= 0;
		request_reqid			<= 0;
		request_tag				<= 0;
		request_lbe				<= 0;
		request_fbe				<= 0;

		sop_plus_one			<= 0;
		sop_plus_two			<= 0;
		reg_master_address		<= 0;

	end else begin

		if( asi_tlpsnk_ready & asi_tlpsnk_valid ) begin

			if( asi_tlpsnk_startofpacket ) begin
			
				request_fmt		<= tlp_fmt_field;
				request_tc		<= tlp_tc_field;
				request_attr	<= tlp_attr_field;
				request_length	<= tlp_length_field;
				request_reqid	<= tlp_reqid_field;
				request_tag		<= tlp_tag_field;
				request_lbe		<= tlp_lbe_field;
				request_fbe		<= tlp_fbe_field;
			end
			
			sop_plus_one	<= asi_tlpsnk_startofpacket;
			sop_plus_two	<= sop_plus_one;
			
			if( sop_plus_one ) begin
			
				reg_master_address		<= { tlp_address_field, 2'b00 };
			end
		end	
	end
end

endmodule
