//----------------------------------------------------------------------------------------------------
//-- (c)2010 Altera Corporation. All rights reserved.
//--
//-- Altera products are protected under numerous U.S. and foreign patents,
//-- maskwork rights, copyrights and other intellectual property laws.
//--
//-- This reference design file, and your use thereof, is subject to and governed
//-- by the terms and conditions of the applicable Altera Reference Design License
//-- Agreement (either as signed by you or found at www.altera.com).  By using
//-- this reference design file, you indicate your acceptance of such terms and
//-- conditions between you and Altera Corporation.  In the event that you do not
//-- agree with such terms and conditions, you may not use the reference design
//-- file and please promptly destroy any copies you have made.
//--
//-- This reference design file is being provided on an as-is basis and as an
//-- accommodation and therefore all warranties, representations or guarantees of
//-- any kind (whether express, implied or statutory) including, without
//-- limitation, warranties of merchantability, non-infringement, or fitness for
//-- a particular purpose, are specifically disclaimed.  By making this reference
//-- design file available, Altera expressly does not recommend, suggest or
//-- require that this reference design file be used in combination with any
//-- other product not provided by Altera.
//----------------------------------------------------------------------------------------------------

//
//	This module provides an endian swap to allow Avalon MM onchip memory FIFO components to receive and
//	transmit raw TLP packets thru the Modular PCIe Bridge.
//
module pcie_bridge_endian_swap_64b
(
	// Clock Interface
	input 							csi_clock_clk,
	input 							csi_clock_reset_n,

	// Avalon ST Sink - in
	input		[63:0]				asi_in_data,
	input							asi_in_valid,
	output	reg						asi_in_ready,
	input							asi_in_startofpacket,
	input							asi_in_endofpacket,

	// Avalon ST Source - out
	output	reg	[63:0]				aso_out_data,
	output	reg						aso_out_valid,
	input							aso_out_ready,
	output	reg						aso_out_startofpacket,
	output	reg						aso_out_endofpacket
);

reg	[7:0]	symbol_0;
reg	[7:0]	symbol_1;
reg	[7:0]	symbol_2;
reg	[7:0]	symbol_3;
reg	[7:0]	symbol_4;
reg	[7:0]	symbol_5;
reg	[7:0]	symbol_6;
reg	[7:0]	symbol_7;

always @ (*) begin

	symbol_0	<= asi_in_data[63:56];
	symbol_1	<= asi_in_data[55:48];
	symbol_2	<= asi_in_data[47:40];
	symbol_3	<= asi_in_data[39:32];
	symbol_4	<= asi_in_data[31:24];
	symbol_5	<= asi_in_data[23:16];
	symbol_6	<= asi_in_data[15:8];
	symbol_7	<= asi_in_data[7:0];
	
	aso_out_data	<= { symbol_7, symbol_6, symbol_5, symbol_4, symbol_3, symbol_2, symbol_1, symbol_0 };
	
	aso_out_valid	<= asi_in_valid;
	asi_in_ready	<= aso_out_ready;
	
	aso_out_startofpacket	<= asi_in_startofpacket;
	aso_out_endofpacket		<= asi_in_endofpacket;
	
end

endmodule
