//----------------------------------------------------------------------------------------------------
//-- (c)2010 Altera Corporation. All rights reserved.
//--
//-- Altera products are protected under numerous U.S. and foreign patents,
//-- maskwork rights, copyrights and other intellectual property laws.
//--
//-- This reference design file, and your use thereof, is subject to and governed
//-- by the terms and conditions of the applicable Altera Reference Design License
//-- Agreement (either as signed by you or found at www.altera.com).  By using
//-- this reference design file, you indicate your acceptance of such terms and
//-- conditions between you and Altera Corporation.  In the event that you do not
//-- agree with such terms and conditions, you may not use the reference design
//-- file and please promptly destroy any copies you have made.
//--
//-- This reference design file is being provided on an as-is basis and as an
//-- accommodation and therefore all warranties, representations or guarantees of
//-- any kind (whether express, implied or statutory) including, without
//-- limitation, warranties of merchantability, non-infringement, or fitness for
//-- a particular purpose, are specifically disclaimed.  By making this reference
//-- design file available, Altera expressly does not recommend, suggest or
//-- require that this reference design file be used in combination with any
//-- other product not provided by Altera.
//----------------------------------------------------------------------------------------------------

//	
//	This module receives the raw tl_cfg_ctl signals asynchronously from the PCIe HIP core and
//	synchronizes them to the local clock domain.  The module then extracts the desired fields from the
//	continuously rotating outputs.
//	
//	The format of the input vector asi_tl_cfg_ctl_in_data is as follows:
//	{ tl_cfg_ctl_wr, tl_cfg_add[ 3 : 0 ], tl_cfg_ctl[ 31 : 0 ] } = asi_tl_cfg_ctl_in_data;
//	
//	The currently extracted fields are as follows:
//	
//	Address |   Field     |      Bits
//	---------------------------------------------
//	  0x0   | cfg_devcsr  | tl_cfg_ctl[ 31 : 16 ]
//	  0x2   | cfg_linkscr | tl_cfg_ctl[ 31 : 16 ]
//	  0xD   | cfg_msicsr  | tl_cfg_ctl[ 15 :  0 ]
//	  0xE   | cfg_tcvcmap | tl_cfg_ctl[ 23 :  0 ]
//	  0xF   | cfg_busdev  | tl_cfg_ctl[ 12 :  0 ]
//	  
	
module pcie_tl_cfg_ctl (

	// Clock Interface
	input wire				clk,
	input wire				reset,
	
	// RAW Input Interface - unsynchronized Avalon ST
	input wire	[ 36 : 0 ]	asi_tl_cfg_ctl_in_data,
	
	// cfg_devcsr Output Interface - Avalon ST
	output reg	[ 15 : 0 ]	aso_cfg_devcsr_out_data,
	
	// cfg_linkscr Output Interface - Avalon ST
	output reg	[ 15 : 0 ]	aso_cfg_linkscr_out_data,
	
	// cfg_busdev Output Interface - Avalon ST
	output reg	[ 12 : 0 ]	aso_cfg_busdev_out_data,
	
	// cfg_msicsr Output Interface - Avalon ST
	output reg	[ 15 : 0 ]	aso_cfg_msicsr_out_data,

	// cfg_tcvcmap Output Interface - Avalon ST
	output reg	[ 23 : 0 ]	aso_cfg_tcvcmap_out_data
);

wire [ 31 : 0 ]	tl_cfg_ctl;
wire [  3 : 0 ]	tl_cfg_add;
wire 			tl_cfg_ctl_wr;
wire 			sync_tl_cfg_ctl_wr;

// command lines for use in TimeQuest to verify the max and min collections
//foreach_in_collection next_fan [ remove_from_collection [ get_fanins -sync *|pcie_tl_cfg_ctl:*|sync_cfg_ctl[*] ] [ get_keepers *|pcie_tl_cfg_ctl:*|* ] ] { puts [ get_object_info -name $next_fan ] }
//foreach_in_collection next_fan [ remove_from_collection [ get_fanins -sync *|pcie_tl_cfg_ctl:*|sync_cfg_add[*] ] [ get_keepers *|pcie_tl_cfg_ctl:*|* ] ] { puts [ get_object_info -name $next_fan ] }
//foreach_in_collection next_fan [ remove_from_collection [ get_fanins -sync *|pcie_tl_cfg_ctl:*|cfg_ctl_wr_s1 ] [ get_keepers *|pcie_tl_cfg_ctl:*|* ] ] { puts [ get_object_info -name $next_fan ] }

//
// These next three registers will capture the asynchronous bus that is output from the HIP core in a
// different clock domain.  So the set_max_delay and set_min_delay are an attempt to keep the skew between
// these signals to less than a half clock at 250MHz.
//
(* altera_attribute = {"-name DONT_MERGE_REGISTER ON; -name PRESERVE_REGISTER ON; -name SDC_STATEMENT \"set_max_delay -from [ remove_from_collection [ get_fanins -sync *|pcie_tl_cfg_ctl:*|sync_cfg_ctl[*] ] [ get_keepers *|pcie_tl_cfg_ctl:*|* ] ] -to [ get_keepers *|pcie_tl_cfg_ctl:*|sync_cfg_ctl[*] ] 2.00 \"; -name SDC_STATEMENT \"set_min_delay -from [ remove_from_collection [ get_fanins -sync *|pcie_tl_cfg_ctl:*|sync_cfg_ctl[*] ] [ get_keepers *|pcie_tl_cfg_ctl:*|* ] ] -to [ get_keepers *|pcie_tl_cfg_ctl:*|sync_cfg_ctl[*] ] 0.00 \" "} *)	reg  [ 31 : 0 ]	sync_cfg_ctl;
(* altera_attribute = {"-name DONT_MERGE_REGISTER ON; -name PRESERVE_REGISTER ON; -name SDC_STATEMENT \"set_max_delay -from [ remove_from_collection [ get_fanins -sync *|pcie_tl_cfg_ctl:*|sync_cfg_add[*] ] [ get_keepers *|pcie_tl_cfg_ctl:*|* ] ] -to [ get_keepers *|pcie_tl_cfg_ctl:*|sync_cfg_add[*] ] 2.00 \"; -name SDC_STATEMENT \"set_min_delay -from [ remove_from_collection [ get_fanins -sync *|pcie_tl_cfg_ctl:*|sync_cfg_add[*] ] [ get_keepers *|pcie_tl_cfg_ctl:*|* ] ] -to [ get_keepers *|pcie_tl_cfg_ctl:*|sync_cfg_add[*] ] 0.00 \""} *)		reg  [  3 : 0 ]	sync_cfg_add;
(* altera_attribute = {"-name DONT_MERGE_REGISTER ON; -name PRESERVE_REGISTER ON; -name SDC_STATEMENT \"set_max_delay -from [ remove_from_collection [ get_fanins -sync *|pcie_tl_cfg_ctl:*|cfg_ctl_wr_s1 ]   [ get_keepers *|pcie_tl_cfg_ctl:*|* ] ] -to [ get_keepers *|pcie_tl_cfg_ctl:*|cfg_ctl_wr_s1 ] 2.00 \";   -name SDC_STATEMENT \"set_min_delay -from [ remove_from_collection [ get_fanins -sync *|pcie_tl_cfg_ctl:*|cfg_ctl_wr_s1 ]   [ get_keepers *|pcie_tl_cfg_ctl:*|* ] ] -to [ get_keepers *|pcie_tl_cfg_ctl:*|cfg_ctl_wr_s1 ] 0.00 \""} *)		reg 			cfg_ctl_wr_s1;

(* altera_attribute = {"-name DONT_MERGE_REGISTER ON; -name PRESERVE_REGISTER ON"} *)	reg 			cfg_ctl_wr_s2;
(* altera_attribute = {"-name DONT_MERGE_REGISTER ON; -name PRESERVE_REGISTER ON"} *)	reg 			sync_cfg_ctl_wr;
reg				take_new_data_now;

assign { tl_cfg_ctl_wr, tl_cfg_add[ 3 : 0 ], tl_cfg_ctl[ 31 : 0 ] } = asi_tl_cfg_ctl_in_data;

//
// Can't use the altera_std_synchronizer since it false paths the input and we need the set_max_delay and set_min_delay constraints to stand
//
//(* altera_attribute = {"-name SDC_STATEMENT \"set_false_path -to [ get_pins { *|*.tl_cfg_ctl_wr_synchronizer_inst|dreg*|clrn } ]\" "} *)
//altera_std_synchronizer
//#(
//    .depth  ( 2 )
//)
//tl_cfg_ctl_wr_synchronizer_inst
//(
//    .clk        (clk),
//    .reset_n    (~reset),
//    .din        (tl_cfg_ctl_wr),
//    .dout       (sync_tl_cfg_ctl_wr)
//);

//
// Create a 2 register synchronizer to synchronize the cfg_ctl_wr strobe.  This cannot be longer than 2
// registers since the worst case toggle rate of cfg_ctl_wr may be 4 clocks.
//
always @ ( posedge clk or posedge reset ) begin
	if( reset ) begin

		cfg_ctl_wr_s1 <= 0;
		cfg_ctl_wr_s2 <= 0;

	end else begin

		cfg_ctl_wr_s1 <= tl_cfg_ctl_wr;
		cfg_ctl_wr_s2 <= cfg_ctl_wr_s1;

	end
end

assign sync_tl_cfg_ctl_wr = cfg_ctl_wr_s2;

always @ ( posedge clk or posedge reset ) begin
	if( reset ) begin
	
		sync_cfg_ctl_wr		<= 0;
		sync_cfg_ctl		<= 0;
		sync_cfg_add		<= 0;
		take_new_data_now	<= 0;
		
	end else begin

		sync_cfg_ctl_wr <= sync_tl_cfg_ctl_wr;
		
		if( sync_cfg_ctl_wr != sync_tl_cfg_ctl_wr ) begin
		
			sync_cfg_ctl <= tl_cfg_ctl;
			sync_cfg_add <= tl_cfg_add;
			
			take_new_data_now <= 1'b1;
			
		end else begin
		
			take_new_data_now <= 1'b0;

		end
		
		if( take_new_data_now ) begin
		
			if( sync_cfg_add == 4'h0 ) begin
			
				aso_cfg_devcsr_out_data <= sync_cfg_ctl[ 31 : 16 ];
				
			end
			
			if( sync_cfg_add == 4'h2 ) begin
			
				aso_cfg_linkscr_out_data <= sync_cfg_ctl[ 31 : 16 ];
				
			end
			
			if( sync_cfg_add == 4'hD ) begin

				aso_cfg_msicsr_out_data <= sync_cfg_ctl[ 15 :  0 ];

			end
			
			if( sync_cfg_add == 4'hE ) begin

				aso_cfg_tcvcmap_out_data <= sync_cfg_ctl[ 23 :  0 ];

			end
			
			if( sync_cfg_add == 4'hF ) begin

				aso_cfg_busdev_out_data <= sync_cfg_ctl[ 12 :  0 ];

			end
			
		end

	end
end

endmodule
