//----------------------------------------------------------------------------------------------------
//-- (c)2010 Altera Corporation. All rights reserved.
//--
//-- Altera products are protected under numerous U.S. and foreign patents,
//-- maskwork rights, copyrights and other intellectual property laws.
//--
//-- This reference design file, and your use thereof, is subject to and governed
//-- by the terms and conditions of the applicable Altera Reference Design License
//-- Agreement (either as signed by you or found at www.altera.com).  By using
//-- this reference design file, you indicate your acceptance of such terms and
//-- conditions between you and Altera Corporation.  In the event that you do not
//-- agree with such terms and conditions, you may not use the reference design
//-- file and please promptly destroy any copies you have made.
//--
//-- This reference design file is being provided on an as-is basis and as an
//-- accommodation and therefore all warranties, representations or guarantees of
//-- any kind (whether express, implied or statutory) including, without
//-- limitation, warranties of merchantability, non-infringement, or fitness for
//-- a particular purpose, are specifically disclaimed.  By making this reference
//-- design file available, Altera expressly does not recommend, suggest or
//-- require that this reference design file be used in combination with any
//-- other product not provided by Altera.
//----------------------------------------------------------------------------------------------------

//	
//	This module simply pipelines the readdata stream and reports back to the tlp2master peripheral each
//	clock cycle that data moves out of this pipeline, so it may manage the read credit counter that it
//	maintains.
//	

module readdata_credit_reporter_64b
(
	// Clock Interface
	input  wire				clk,
	input  wire				reset,
	
	// Read Data Input Interface - Avalon ST
	input  wire	[ 63 : 0 ]	asi_rddatasnk_data,
	output wire				asi_rddatasnk_ready,
	input  wire				asi_rddatasnk_valid,

	// Read Data Output Interface - Avalon ST
	output wire	[ 63 : 0 ]	aso_rddatasrc_data,
	input  wire				aso_rddatasrc_ready,
	output wire				aso_rddatasrc_valid,
	
	// Credit Decrement Report Interface - Avalon ST
	output wire				aso_readcreditdec_data
);

reg	read_credit_dec;

always @ ( posedge clk or posedge reset ) begin

	if( reset ) begin
	
		read_credit_dec <= 0;
		
	end else begin
		
		read_credit_dec <= aso_rddatasrc_ready & aso_rddatasrc_valid;
	end
end

assign aso_readcreditdec_data = read_credit_dec;

//
// readdata output Pipeline
//
readdata_credit_reporter_64b_1stage_pipeline #( .PAYLOAD_WIDTH( 64 ) ) readdata_outpipe (
    .clk            ( clk ),
    .reset_n        ( ~reset ),
    .in_ready       ( asi_rddatasnk_ready ),
    .in_valid       ( asi_rddatasnk_valid ), 
    .in_payload     ( asi_rddatasnk_data ),
    .out_ready      ( aso_rddatasrc_ready ), 
    .out_valid      ( aso_rddatasrc_valid ), 
    .out_payload    ( aso_rddatasrc_data )
);

endmodule

//  --------------------------------------------------------------------------------
// | single buffered pipeline stage
//  --------------------------------------------------------------------------------
module readdata_credit_reporter_64b_1stage_pipeline  
#( parameter PAYLOAD_WIDTH = 8 )
 ( input                          clk,
   input                          reset_n, 
   output reg                     in_ready,
   input                          in_valid,   
   input      [PAYLOAD_WIDTH-1:0] in_payload,
   input                          out_ready,   
   output reg                     out_valid,
   output reg [PAYLOAD_WIDTH-1:0] out_payload      
 );
      
   always @* begin
     in_ready <= out_ready || ~out_valid;
   end
   
   always @ (negedge reset_n, posedge clk) begin
      if (!reset_n) begin
         out_valid <= 0;
         out_payload <= 0;
      end else begin
         if (in_valid) begin
           out_valid <= 1;
         end else if (out_ready) begin
           out_valid <= 0;
         end
         
         if(in_valid && in_ready) begin
            out_payload <= in_payload;
         end
      end
   end

endmodule
