//----------------------------------------------------------------------------------------------------
//-- (c)2010 Altera Corporation. All rights reserved.
//--
//-- Altera products are protected under numerous U.S. and foreign patents,
//-- maskwork rights, copyrights and other intellectual property laws.
//--
//-- This reference design file, and your use thereof, is subject to and governed
//-- by the terms and conditions of the applicable Altera Reference Design License
//-- Agreement (either as signed by you or found at www.altera.com).  By using
//-- this reference design file, you indicate your acceptance of such terms and
//-- conditions between you and Altera Corporation.  In the event that you do not
//-- agree with such terms and conditions, you may not use the reference design
//-- file and please promptly destroy any copies you have made.
//--
//-- This reference design file is being provided on an as-is basis and as an
//-- accommodation and therefore all warranties, representations or guarantees of
//-- any kind (whether express, implied or statutory) including, without
//-- limitation, warranties of merchantability, non-infringement, or fitness for
//-- a particular purpose, are specifically disclaimed.  By making this reference
//-- design file available, Altera expressly does not recommend, suggest or
//-- require that this reference design file be used in combination with any
//-- other product not provided by Altera.
//----------------------------------------------------------------------------------------------------

//	
//	This module acts as a full packet wait gate, with the additional ability to correct the length field
//	and the lbe feild of the TLP header for write requests.
//	

module request_packet_wait_gate_64b
(
	// Clock Interface
	input  wire				clk,
	input  wire				reset,
	
	// Input Interface - Avalon ST
	input  wire	[ 63 : 0 ]	asi_snk_data,
	output wire				asi_snk_ready,
	input  wire				asi_snk_valid,
	input  wire				asi_snk_startofpacket,
	input  wire				asi_snk_endofpacket,

	// EOPLBE Input Interface - Avalon ST
	input  wire	[  4 : 0 ]	asi_eoplbesnk_data,
	output wire				asi_eoplbesnk_ready,
	input  wire				asi_eoplbesnk_valid,

	// Output Interface - Avalon ST
	output wire	[ 63 : 0 ]	aso_src_data,
	input  wire				aso_src_ready,
	output wire				aso_src_valid,
	output wire				aso_src_startofpacket,
	output wire				aso_src_endofpacket
);

wire in_ready;
wire in_valid;
wire [ 63 : 0 ] pipe_payload_data;

//
// Output TLP header fields
//
wire [ 63 : 0 ]	output_tlp_header_1_0;
wire [  9 : 0 ] output_tlp_length_field;
wire [  3 : 0 ] output_tlp_lbe_field;


//
// TLP header word 0
//
wire [  0 : 0 ] tlp_res3_field;
wire [  1 : 0 ] tlp_fmt_field;
wire [  4 : 0 ] tlp_type_field;
wire [  0 : 0 ] tlp_res2_field;
wire [  2 : 0 ] tlp_tc_field;
wire [  3 : 0 ] tlp_res1_field;
wire [  0 : 0 ] tlp_td_field;
wire [  0 : 0 ] tlp_ep_field;
wire [  1 : 0 ] tlp_attr_field;
wire [  1 : 0 ] tlp_res0_field;
wire [  9 : 0 ] tlp_length_field;

//
// TLP header word 1
//
wire [ 15 : 0 ] tlp_reqid_field;
wire [  7 : 0 ] tlp_tag_field;
wire [  3 : 0 ] tlp_lbe_field;
wire [  3 : 0 ] tlp_fbe_field;

assign {
			tlp_reqid_field,
			tlp_tag_field,
			tlp_lbe_field,
			tlp_fbe_field,
			tlp_res3_field,
			tlp_fmt_field,
			tlp_type_field,
			tlp_res2_field,
			tlp_tc_field,
			tlp_res1_field,
			tlp_td_field,
			tlp_ep_field,
			tlp_attr_field,
			tlp_res0_field,
			tlp_length_field
							} = asi_snk_data;
							
assign output_tlp_header_1_0 = {
									tlp_reqid_field,
									tlp_tag_field,
									output_tlp_lbe_field,
									tlp_fbe_field,
									tlp_res3_field,
									tlp_fmt_field,
									tlp_type_field,
									tlp_res2_field,
									tlp_tc_field,
									tlp_res1_field,
									tlp_td_field,
									tlp_ep_field,
									tlp_attr_field,
									tlp_res0_field,
									output_tlp_length_field
								};

assign output_tlp_length_field =	( (tlp_fmt_field[1] == 1 ) && ( asi_eoplbesnk_data[4] == 1 ) ) ?	// write cycle - sub1dw?
										( tlp_length_field - 1 ) :											// write & sub1dw - subtract 1 from length
										( tlp_length_field );												// otherwise - keep original length

assign output_tlp_lbe_field =	(tlp_fmt_field[1] == 1 ) ? 				// write cycle?
									( asi_eoplbesnk_data[ 3 : 0] ) :		// write - substitute lbe field
									( tlp_lbe_field );						// not write - keep lbe field

assign pipe_payload_data =	( asi_snk_startofpacket ) ?			// startofpacket?
								( output_tlp_header_1_0 ) : 		// sop - output header_1_0
								( asi_snk_data );					// !sop - output snk_data

always @ ( posedge clk or posedge reset ) begin

	if( reset ) begin
	
	end else begin
		
	end
end

assign asi_eoplbesnk_ready = asi_snk_startofpacket & asi_snk_ready & in_valid;

assign asi_snk_ready =	( asi_snk_startofpacket ) ?		// sop?
							( asi_eoplbesnk_valid ) ? 		// sop - eoplbe valid?
								( in_ready ) : 					// eoplbe valid - drive in_ready out sink interface
								( 1'b0 ) :						// !eoplbe valid - drive 1'b0 out sink interface
							( in_ready );					// !sop - drive in_ready out sink interface

assign in_valid =	( asi_snk_startofpacket ) ?		// sop?
						( asi_eoplbesnk_valid ) ? 		// sop - eoplbe valid?
							( asi_snk_valid ) : 			// eoplbe valid - drive asi_snk_valid to pipeline output stage
							( 1'b0 ) :						// !eoplbe valid - drive 1'b0 to pipeline output stage
						( asi_snk_valid );				// !sop - drive asi_snk_valid to pipeline output stage

//
// readdata output Pipeline
//
request_packet_wait_gate_64b_1stage_pipeline #( .PAYLOAD_WIDTH( 66 ) ) outpipe (
    .clk            ( clk ),
    .reset_n        ( ~reset ),
    .in_ready       ( in_ready ),
    .in_valid       ( in_valid ), 
    .in_payload     ( { pipe_payload_data, asi_snk_startofpacket, asi_snk_endofpacket } ),
    .out_ready      ( aso_src_ready ), 
    .out_valid      ( aso_src_valid ), 
    .out_payload    ( { aso_src_data, aso_src_startofpacket, aso_src_endofpacket } )
);

endmodule

//  --------------------------------------------------------------------------------
// | single buffered pipeline stage
//  --------------------------------------------------------------------------------
module request_packet_wait_gate_64b_1stage_pipeline  
#( parameter PAYLOAD_WIDTH = 8 )
 ( input                          clk,
   input                          reset_n, 
   output reg                     in_ready,
   input                          in_valid,   
   input      [PAYLOAD_WIDTH-1:0] in_payload,
   input                          out_ready,   
   output reg                     out_valid,
   output reg [PAYLOAD_WIDTH-1:0] out_payload      
 );
      
   always @* begin
     in_ready <= out_ready || ~out_valid;
   end
   
   always @ (negedge reset_n, posedge clk) begin
      if (!reset_n) begin
         out_valid <= 0;
         out_payload <= 0;
      end else begin
         if (in_valid) begin
           out_valid <= 1;
         end else if (out_ready) begin
           out_valid <= 0;
         end
         
         if(in_valid && in_ready) begin
            out_payload <= in_payload;
         end
      end
   end

endmodule
