#!/bin/bash

#
# Start by validating the input arguments to the script
#
VALID_ARGS=0
SRC_DIR=""
if [ $# -ne 1 ] ; then
	echo ""
	echo "ERROR: Wrong number of arguments passed into script..."
	echo ""
	VALID_ARGS=1
else
	SRC_DIR=$(basename ${1})

	if [ ! -d ${SRC_DIR} ] ; then
		echo ""
		echo "ERROR: directory argument does not define a valid directory..."
		echo "requested directory was:"
		echo "\"${SRC_DIR}\""
		echo ""
		VALID_ARGS=1
	fi
fi

if [ ${VALID_ARGS} -ne 0 ] ; then
	echo ""
	echo "USAGE: ./make_targz_ball.zh <target-directory>"
	echo "<target-directory> - is the directory that you wish to archive"
	echo ""

	exit 1
fi

echo ""
echo "Processing directory:"
echo "\"${SRC_DIR}\""
echo ""

#
# Gather all the source files and make them UNIX formatted text files
#
SRC_FILES=$(find ${SRC_DIR} -type 'f')

dos2unix ${SRC_FILES} 2> /dev/null

#
# Create a date stamped name for our archive
#
PREFIX=$(date +%Y%m%d_)
SUFFIX=".tgz"

ARCHIVE_NAME="${PREFIX}${SRC_DIR}${SUFFIX}"

#
# Check if our archive name already exists, if so backup the existing archive
# with an increment value
#
if [ -e ${ARCHIVE_NAME} ] ; then
	echo "Archive already exists:"
	echo "\"${ARCHIVE_NAME}\""
	echo ""

	NEXT_INCREMENT=1
	NEXT_NAME="${PREFIX}~${NEXT_INCREMENT}_${SRC_DIR}${SUFFIX}"
	while [ -e ${NEXT_NAME} ] ; do
		NEXT_INCREMENT=$(expr ${NEXT_INCREMENT} + 1)
		NEXT_NAME="${PREFIX}~${NEXT_INCREMENT}_${SRC_DIR}${SUFFIX}"
	done

	echo "moving archive:"
	echo "\"${ARCHIVE_NAME}\""
	echo "to backup:"
	echo "\"${NEXT_NAME}\""
	echo ""

	mv ${ARCHIVE_NAME} ${NEXT_NAME}
fi

echo "Creating archive:"
echo "\"${ARCHIVE_NAME}\""
echo ""

#
# Make the archive
#
tar -czf ${ARCHIVE_NAME} ${SRC_FILES}

exit 0
