module register_wrapper
#(
	parameter INSIG_WIDTH  = 0,
	parameter OUTSIG_WIDTH = 0
)
(
	clk,
	reset_n,

	insig_in,
	outsig_out
);

localparam AVALON_MASTER_ADDRESS_WIDTH = 31;
localparam COMPLETION_INFO_WIDTH = 64;

input	wire							clk;
input	wire							reset_n;

input	[  ( INSIG_WIDTH - 1 ) : 0 ]	insig_in;
output	[ ( OUTSIG_WIDTH - 1 ) : 0 ]	outsig_out;

reg		[  ( INSIG_WIDTH - 1 ) : 0 ]	reg_in;
reg		[ ( OUTSIG_WIDTH - 1 ) : 0 ]	reg_out;

wire	[  ( INSIG_WIDTH - 1 ) : 0 ]	in_vect;
wire	[ ( OUTSIG_WIDTH - 1 ) : 0 ]	out_vect;

reg										my_reset_n;

//
// instance input signal wires
//
wire  clk_sig;
wire  reset_sig;
wire [63:0] asi_tlpsnk_data_sig;
wire  asi_tlpsnk_valid_sig;
wire  asi_tlpsnk_startofpacket_sig;
wire  asi_tlpsnk_endofpacket_sig;
wire  aso_cmdsrc_ready_sig;
wire  aso_cplinfosrc_ready_sig;
wire  asi_readcreditdec_data_sig;

//
// instance output signal wires
//
wire  asi_tlpsnk_ready_sig;
wire [AVALON_MASTER_ADDRESS_WIDTH+83-1:0] aso_cmdsrc_data_sig;
wire  aso_cmdsrc_valid_sig;
wire [COMPLETION_INFO_WIDTH-1:0] aso_cplinfosrc_data_sig;
wire  aso_cplinfosrc_valid_sig;

//
// assign clock and reset drivers
//
assign clk_sig = clk;
//assign FIXME_reset_n_sig = my_reset_n;	//  my_reset_n for negative asserted reset
assign reset_sig = ~my_reset_n;		// ~my_reset_n for positive asserted reset

//
// map input and output registers to input and output vectors
//
assign in_vect = reg_in;
assign outsig_out = reg_out;

//
// map input and output vectors to instance inputs and outputs
//
assign {
	asi_tlpsnk_data_sig,
	asi_tlpsnk_valid_sig,
	asi_tlpsnk_startofpacket_sig,
	asi_tlpsnk_endofpacket_sig,
	aso_cmdsrc_ready_sig,
	aso_cplinfosrc_ready_sig,
	asi_readcreditdec_data_sig
} = in_vect;

assign out_vect = {
	asi_tlpsnk_ready_sig,
	aso_cmdsrc_data_sig,
	aso_cmdsrc_valid_sig,
	aso_cplinfosrc_data_sig,
	aso_cplinfosrc_valid_sig
};

always @ ( posedge clk ) begin
	
		my_reset_n <= reset_n;
end

always @ ( posedge clk or negedge my_reset_n ) begin
	
	if( ~my_reset_n ) begin
	
		reg_in <= 0;
		reg_out <= 0;
		
	end else begin
	
		reg_in <= insig_in;
		reg_out <= out_vect;

	end
end

//
// instanctiate test instance here
//
pcie_64bar_tlp2master_64b pcie_64bar_tlp2master_64b_inst
(
	.clk(clk_sig) ,	// input  clk_sig
	.reset(reset_sig) ,	// input  reset_sig
	.asi_tlpsnk_data(asi_tlpsnk_data_sig) ,	// input [63:0] asi_tlpsnk_data_sig
	.asi_tlpsnk_ready(asi_tlpsnk_ready_sig) ,	// output  asi_tlpsnk_ready_sig
	.asi_tlpsnk_valid(asi_tlpsnk_valid_sig) ,	// input  asi_tlpsnk_valid_sig
	.asi_tlpsnk_startofpacket(asi_tlpsnk_startofpacket_sig) ,	// input  asi_tlpsnk_startofpacket_sig
	.asi_tlpsnk_endofpacket(asi_tlpsnk_endofpacket_sig) ,	// input  asi_tlpsnk_endofpacket_sig
	.aso_cmdsrc_data(aso_cmdsrc_data_sig) ,	// output [AVALON_MASTER_ADDRESS_WIDTH+83-1:0] aso_cmdsrc_data_sig
	.aso_cmdsrc_ready(aso_cmdsrc_ready_sig) ,	// input  aso_cmdsrc_ready_sig
	.aso_cmdsrc_valid(aso_cmdsrc_valid_sig) ,	// output  aso_cmdsrc_valid_sig
	.aso_cplinfosrc_data(aso_cplinfosrc_data_sig) ,	// output [COMPLETION_INFO_WIDTH-1:0] aso_cplinfosrc_data_sig
	.aso_cplinfosrc_ready(aso_cplinfosrc_ready_sig) ,	// input  aso_cplinfosrc_ready_sig
	.aso_cplinfosrc_valid(aso_cplinfosrc_valid_sig) ,	// output  aso_cplinfosrc_valid_sig
	.asi_readcreditdec_data(asi_readcreditdec_data_sig) 	// input  asi_readcreditdec_data_sig
);

defparam pcie_64bar_tlp2master_64b_inst.AVALON_MASTER_ADDRESS_WIDTH = AVALON_MASTER_ADDRESS_WIDTH;
defparam pcie_64bar_tlp2master_64b_inst.COMPLETION_INFO_WIDTH = COMPLETION_INFO_WIDTH;
defparam pcie_64bar_tlp2master_64b_inst.READ_CREDIT_WIDTH = 10;

endmodule
