//change this file for different test case
initial
begin
  @ start_test;
  
//
//	Read all valid byte combinations thru an entire 64-bit word
//
	burst_count = 1;	// signal burst 1 word
	burst_size = 1;		// and burst 1 word
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	command_addr = `PCIE_32BIT_TEST_ADDR_OFFSET;
	command_data = 64'h0123_4567_89AB_CDEF;

	for ( int j = 0 ; j < 8 ; j++ ) begin

		byte_enable = 8'hFF >> ( 7 - j );
		command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF >> ( ( 7 - j ) * 8 );
		for ( int i = 0 ; i < ( 8 - j ) ; i++ ) begin
		
			master_data_scoreboard.push_back( command_data );
			master_mask_scoreboard.push_back( command_data_mask );
			
			master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
			
			command_data_mask = command_data_mask << 8;
			byte_enable = byte_enable << 1;
		end
	end

//
//	Read 2 64-bit word burst
//
	burst_count = 2;	// signal burst 2 word
	burst_size = 2;		// and burst 2 word
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	
	command_addr = `PCIE_32BIT_TEST_ADDR_OFFSET;
	byte_enable = 8'hFF;

	command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF;	
	command_data = 64'h0123_4567_89AB_CDEF;
	master_data_scoreboard.push_back( command_data );
	master_mask_scoreboard.push_back( command_data_mask );

	command_data = 64'hFEDC_BA98_7654_3210;
	master_data_scoreboard.push_back( command_data );
	master_mask_scoreboard.push_back( command_data_mask );

	command_data = 64'h0;
	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);

//
//	Read aligned large burst blocks from 16 byte thru 4096 byte
//
	command_addr = ( 4 * 1024 ) + `PCIE_32BIT_TEST_ADDR_OFFSET;
	byte_enable = 8'hFF;
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	
	burst_count = ( 16 / 8 );
	burst_size = burst_count;

	command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF;
	
	while( burst_count > 0 ) begin
		command_data = 64'h0;
		for ( int i = 0 ; i < burst_count ; i++ ) begin
			master_data_scoreboard.push_back( command_data );
			master_mask_scoreboard.push_back( command_data_mask );
			command_data = command_data + 1;
		end

		command_data = 64'h0;
		master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
		burst_count = burst_count << 1;
		burst_size = burst_count;
	end

//
//	Read unaligned +8 large burst blocks from 16 byte thru 4096 byte
//
	command_addr = ( 8 + ( 8 * 1024 ) ) + `PCIE_32BIT_TEST_ADDR_OFFSET;
	byte_enable = 8'hFF;
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	
	burst_count = ( 16 / 8 );
	burst_size = burst_count;

	command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF;
	
	while( burst_count > 0 ) begin
		command_data = 64'd513 - burst_count;
		for ( int i = 0 ; i < burst_count ; i++ ) begin
			master_data_scoreboard.push_back( command_data );
			master_mask_scoreboard.push_back( command_data_mask );
			command_data = command_data + 1;
		end

		command_data = 64'h0;
		master_set_and_push_command(REQ_READ, command_addr - ( burst_count * 8 ), command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
		burst_count = burst_count << 1;
		burst_size = burst_count;
	end

//
//	Read unaligned -8 large burst blocks from 16 byte thru 4096 byte
//
	command_addr = ( ( 8 * 1024 ) - 8 ) + `PCIE_32BIT_TEST_ADDR_OFFSET;
	byte_enable = 8'hFF;
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	
	burst_count = ( 16 / 8 );
	burst_size = burst_count;

	command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF;
	
	while( burst_count > 0 ) begin
		command_data = 64'd511;
		for ( int i = 0 ; i < burst_count ; i++ ) begin
			master_data_scoreboard.push_back( command_data );
			master_mask_scoreboard.push_back( command_data_mask );
			command_data = command_data + 1;
		end

		command_data = 64'h0;
		master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
		burst_count = burst_count << 1;
		burst_size = burst_count;
	end

//
//	Begin single word write tests
//
	index_4k = 0;
	command_addr = ( index_4k * ( 4 * 1024 ) ) + ( 16 * 1024 ) + `PCIE_32BIT_TEST_ADDR_OFFSET;
	command_addr_base = command_addr;

//
//	Write all valid byte combinations thru an entire 64-bit word
//
	burst_count = 1;	// signal burst 1 word
	burst_size = 1;		// and burst 1 word
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	command_data = 64'h8786858483828180;
	
	for( int j = 0 ; j < 8 ; j++ ) begin

		byte_enable = 8'hFF >> j;
		command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF >> ( j * 8 );
		for ( int i = 0 ; i < ( j + 1 ) ; i++ ) begin
		
			master_data_scoreboard.push_back( command_data );
			master_mask_scoreboard.push_back( command_data_mask );
			
			master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
			
			command_data_mask = command_data_mask << 8;
			byte_enable = byte_enable << 1;
			
			command_addr = command_addr + 8;
		end
		command_data = command_data - 64'h1010_1010_1010_1010;
	end
//
//	Read back and validate the write data to this point
//
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	
	burst_count = ( command_addr - command_addr_base ) / 8;
	burst_size = burst_count;

	byte_enable = 8'hFF;
	command_data = 64'h0;

	master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
//
//	Begin multiple word write burst tests - aligned
//
	for( index_4k = 1 ; index_4k < 8 ; index_4k++ ) begin
		command_addr = ( index_4k * ( 4 * 1024 ) ) + ( 16 * 1024 ) + `PCIE_32BIT_TEST_ADDR_OFFSET;
		command_addr_base = command_addr;

	//
	//	Write 2 thru 8 word bursts with varied byte enables
	//
		burst_count = index_4k + 1;
		burst_size = burst_count;
		idle = 0;  			// no idle cycle between each command of the master BFM
		init_latency = 0;  	// the command is launched to Avalon bus with no delay
		
		byte_enable = 8'hFF;
		last_byte_enable = 8'hFF;
		for ( int i = 0 ; i < 64 ; i++ ) begin
		

			master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
			
			byte_enable = byte_enable << 1;
			last_byte_enable = last_byte_enable >> 1;
			
			if( byte_enable == 0 ) begin

				byte_enable = 8'hFF;
				last_byte_enable = 8'hFF;
			end
			
			command_addr = command_addr + ( burst_count * 8 );
		end

	//
	//	Read back and validate the write data to this point
	//
		idle = 0;  			// no idle cycle between each command of the master BFM
		init_latency = 0;  	// the command is launched to Avalon bus with no delay
		
		burst_count = ( command_addr - command_addr_base ) / 8;
		burst_size = burst_count;

		byte_enable = 8'hFF;
		command_data = 64'h0;

		master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
	end

//
//	Begin multiple word write burst tests - unaligned +8
//
	for( index_4k = 9 ; index_4k < 16 ; index_4k++ ) begin
		command_addr = 8 + ( index_4k * ( 4 * 1024 ) ) + ( 16 * 1024 ) + `PCIE_32BIT_TEST_ADDR_OFFSET;
		command_addr_base = command_addr;

	//
	//	Write 2 thru 8 word bursts with varied byte enables
	//
		burst_count = index_4k - 7;
		burst_size = burst_count;
		idle = 0;  			// no idle cycle between each command of the master BFM
		init_latency = 0;  	// the command is launched to Avalon bus with no delay
		
		command_addr = command_addr - ( burst_count * 8 );
		command_addr_base = command_addr;

		byte_enable = 8'hFF;
		last_byte_enable = 8'hFF;
		for ( int i = 0 ; i < 64 ; i++ ) begin
		

			master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
			
			byte_enable = byte_enable << 1;
			last_byte_enable = last_byte_enable >> 1;
			
			if( byte_enable == 0 ) begin

				byte_enable = 8'hFF;
				last_byte_enable = 8'hFF;
			end
			
			command_addr = command_addr + ( burst_count * 8 );
		end

	//
	//	Read back and validate the write data to this point
	//
		idle = 0;  			// no idle cycle between each command of the master BFM
		init_latency = 0;  	// the command is launched to Avalon bus with no delay
		
		burst_count = ( command_addr - command_addr_base ) / 8;
		burst_size = burst_count;

		byte_enable = 8'hFF;
		command_data = 64'h0;

		master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
	end

//
//	Begin multiple word write burst tests - unaligned -8
//
	for( index_4k = 16 ; index_4k < 23 ; index_4k++ ) begin
		command_addr = ( ( ( index_4k + 1 ) * ( 4 * 1024 ) ) - 8 ) + ( 16 * 1024 ) + `PCIE_32BIT_TEST_ADDR_OFFSET;
		command_addr_base = command_addr;

	//
	//	Write 2 thru 8 word bursts with varied byte enables
	//
		burst_count = index_4k - 14;
		burst_size = burst_count;
		idle = 0;  			// no idle cycle between each command of the master BFM
		init_latency = 0;  	// the command is launched to Avalon bus with no delay
		
		byte_enable = 8'hFF;
		last_byte_enable = 8'hFF;
		for ( int i = 0 ; i < 64 ; i++ ) begin
		

			master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
			
			byte_enable = byte_enable << 1;
			last_byte_enable = last_byte_enable >> 1;
			
			if( byte_enable == 0 ) begin

				byte_enable = 8'hFF;
				last_byte_enable = 8'hFF;
			end
			
			command_addr = command_addr + ( burst_count * 8 );
		end

	//
	//	Read back and validate the write data to this point
	//
		idle = 0;  			// no idle cycle between each command of the master BFM
		init_latency = 0;  	// the command is launched to Avalon bus with no delay
		
		burst_count = ( command_addr - command_addr_base ) / 8;
		burst_size = burst_count;

		byte_enable = 8'hFF;
		command_data = 64'h0;

		master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
	end
	
//
//	Begin large multiple word write burst tests - aligned
//
	index_4k = 24; // use this for next write block

	//
	//	Write 2 thru 512 word bursts with varied byte enables
	//
	burst_count = 2;
	burst_size = burst_count;
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay

	byte_enable = 8'hFF;
	last_byte_enable = 8'hFF;
	
	command_data = 64'hFFFF_FFFF_FFFF_FFFF;
	
	while( burst_count > 0 ) begin
	
		command_addr = ( index_4k * ( 4 * 1024 ) ) + ( 16 * 1024 ) + `PCIE_32BIT_TEST_ADDR_OFFSET;
		command_addr_base = command_addr;

		master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );

	//
	//	Read back and validate the write data to this point
	//
		master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
		burst_count = burst_count << 1;
		burst_size = burst_count;
		index_4k++;
	end

//
//	Begin large multiple word write burst tests - unaligned +8
//
	index_4k = 34; // use this for next write block

	//
	//	Write 2 thru 512 word bursts with varied byte enables
	//
	burst_count = 2;
	burst_size = burst_count;
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay

	byte_enable = 8'hFF;
	last_byte_enable = 8'hFF;
	
	command_data = 64'hFFFF_FFFF_FFFF_FFFF;
	
	while( burst_count > 0 ) begin
	
		command_addr = 8 + ( index_4k * ( 4 * 1024 ) ) + ( 16 * 1024 ) + `PCIE_32BIT_TEST_ADDR_OFFSET;
		command_addr_base = command_addr - ( burst_count * 8 );
		
		master_set_and_push_command_write_burst( command_addr_base, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
			

	//
	//	Read back and validate the write data to this point
	//
		master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
		burst_count = burst_count << 1;
		burst_size = burst_count;
		index_4k++;
	end

//
//	Begin large multiple word write burst tests - unaligned -8
//
	index_4k = 44; // use this for next write block

	//
	//	Write 2 thru 512 word bursts with varied byte enables
	//
	burst_count = 2;
	burst_size = burst_count;
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay

	byte_enable = 8'hFF;
	last_byte_enable = 8'hFF;
	
	command_data = 64'hFFFF_FFFF_FFFF_FFFF;
	
	while( burst_count > 0 ) begin
	
		command_addr = ( ( ( index_4k + 1 ) * ( 4 * 1024 ) ) - 8 ) + ( 16 * 1024 ) + `PCIE_32BIT_TEST_ADDR_OFFSET;
		command_addr_base = command_addr;

		master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
			

	//
	//	Read back and validate the write data to this point
	//
		master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
		burst_count = burst_count << 1;
		burst_size = burst_count;
		index_4k++;
	end

//
// TOGGLE to 64-bit address
//
	wait( checked == request_count );

	burst_count = 1;		// signal burst 1 word
	burst_size = 1;		// and burst 1 word
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay

	command_addr = `SLAVE_0_CSR_BASE;
	byte_enable = 'hFF;
	command_data = 'h0000000100000000;
	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);

//
//	Read all valid byte combinations thru an entire 64-bit word
//
	burst_count = 1;	// signal burst 1 word
	burst_size = 1;		// and burst 1 word
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	command_addr = `PCIE_64BIT_TEST_ADDR_OFFSET;
	command_data = 64'hFEDC_BA98_7654_3210;

	for ( int j = 0 ; j < 8 ; j++ ) begin

		byte_enable = 8'hFF >> ( 7 - j );
		command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF >> ( ( 7 - j ) * 8 );
		for ( int i = 0 ; i < ( 8 - j ) ; i++ ) begin
		
			master_data_scoreboard.push_back( command_data );
			master_mask_scoreboard.push_back( command_data_mask );
			
			master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
			
			command_data_mask = command_data_mask << 8;
			byte_enable = byte_enable << 1;
		end
	end

//
//	Read 2 64-bit word burst
//
	burst_count = 2;	// signal burst 2 word
	burst_size = 2;		// and burst 2 word
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	
	command_addr = `PCIE_64BIT_TEST_ADDR_OFFSET;
	byte_enable = 8'hFF;

	command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF;	
	command_data = 64'hFEDC_BA98_7654_3210;
	master_data_scoreboard.push_back( command_data );
	master_mask_scoreboard.push_back( command_data_mask );

	command_data = 64'h0123_4567_89AB_CDEF;
	master_data_scoreboard.push_back( command_data );
	master_mask_scoreboard.push_back( command_data_mask );

	command_data = 64'h0;
	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);

//
//	Read aligned large burst blocks from 16 byte thru 4096 byte
//
	command_addr = ( 4 * 1024 ) + `PCIE_64BIT_TEST_ADDR_OFFSET;
	byte_enable = 8'hFF;
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	
	burst_count = ( 16 / 8 );
	burst_size = burst_count;

	command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF;
	
	while( burst_count > 0 ) begin
		command_data = 64'h0;
		for ( int i = 0 ; i < burst_count ; i++ ) begin
			master_data_scoreboard.push_back( command_data );
			master_mask_scoreboard.push_back( command_data_mask );
			command_data = command_data + 1;
		end

		command_data = 64'h0;
		master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
		burst_count = burst_count << 1;
		burst_size = burst_count;
	end

//
//	Read unaligned +8 large burst blocks from 16 byte thru 4096 byte
//
	command_addr = ( 8 + ( 8 * 1024 ) ) + `PCIE_64BIT_TEST_ADDR_OFFSET;
	byte_enable = 8'hFF;
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	
	burst_count = ( 16 / 8 );
	burst_size = burst_count;

	command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF;
	
	while( burst_count > 0 ) begin
		command_data = 64'd513 - burst_count;
		for ( int i = 0 ; i < burst_count ; i++ ) begin
			master_data_scoreboard.push_back( command_data );
			master_mask_scoreboard.push_back( command_data_mask );
			command_data = command_data + 1;
		end

		command_data = 64'h0;
		master_set_and_push_command(REQ_READ, command_addr - ( burst_count * 8 ), command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
		burst_count = burst_count << 1;
		burst_size = burst_count;
	end

//
//	Read unaligned -8 large burst blocks from 16 byte thru 4096 byte
//
	command_addr = ( ( 8 * 1024 ) - 8 ) + `PCIE_64BIT_TEST_ADDR_OFFSET;
	byte_enable = 8'hFF;
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	
	burst_count = ( 16 / 8 );
	burst_size = burst_count;

	command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF;
	
	while( burst_count > 0 ) begin
		command_data = 64'd511;
		for ( int i = 0 ; i < burst_count ; i++ ) begin
			master_data_scoreboard.push_back( command_data );
			master_mask_scoreboard.push_back( command_data_mask );
			command_data = command_data + 1;
		end

		command_data = 64'h0;
		master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
		burst_count = burst_count << 1;
		burst_size = burst_count;
	end

//
//	Begin single word write tests
//
	index_4k = 0;
	command_addr = ( index_4k * ( 4 * 1024 ) ) + ( 16 * 1024 ) + `PCIE_64BIT_TEST_ADDR_OFFSET;
	command_addr_base = command_addr;

//
//	Write all valid byte combinations thru an entire 64-bit word
//
	burst_count = 1;	// signal burst 1 word
	burst_size = 1;		// and burst 1 word
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	command_data = 64'h8786858483828180;
	
	for( int j = 0 ; j < 8 ; j++ ) begin

		byte_enable = 8'hFF >> j;
		command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF >> ( j * 8 );
		for ( int i = 0 ; i < ( j + 1 ) ; i++ ) begin
		
			master_data_scoreboard.push_back( command_data );
			master_mask_scoreboard.push_back( command_data_mask );
			
			master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
			
			command_data_mask = command_data_mask << 8;
			byte_enable = byte_enable << 1;
			
			command_addr = command_addr + 8;
		end
		command_data = command_data - 64'h1010_1010_1010_1010;
	end
//
//	Read back and validate the write data to this point
//
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	
	burst_count = ( command_addr - command_addr_base ) / 8;
	burst_size = burst_count;

	byte_enable = 8'hFF;
	command_data = 64'h0;

	master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
//
//	Begin multiple word write burst tests - aligned
//
	for( index_4k = 1 ; index_4k < 8 ; index_4k++ ) begin
		command_addr = ( index_4k * ( 4 * 1024 ) ) + ( 16 * 1024 ) + `PCIE_64BIT_TEST_ADDR_OFFSET;
		command_addr_base = command_addr;

	//
	//	Write 2 thru 8 word bursts with varied byte enables
	//
		burst_count = index_4k + 1;
		burst_size = burst_count;
		idle = 0;  			// no idle cycle between each command of the master BFM
		init_latency = 0;  	// the command is launched to Avalon bus with no delay
		
		byte_enable = 8'hFF;
		last_byte_enable = 8'hFF;
		for ( int i = 0 ; i < 64 ; i++ ) begin
		

			master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
			
			byte_enable = byte_enable << 1;
			last_byte_enable = last_byte_enable >> 1;
			
			if( byte_enable == 0 ) begin

				byte_enable = 8'hFF;
				last_byte_enable = 8'hFF;
			end
			
			command_addr = command_addr + ( burst_count * 8 );
		end

	//
	//	Read back and validate the write data to this point
	//
		idle = 0;  			// no idle cycle between each command of the master BFM
		init_latency = 0;  	// the command is launched to Avalon bus with no delay
		
		burst_count = ( command_addr - command_addr_base ) / 8;
		burst_size = burst_count;

		byte_enable = 8'hFF;
		command_data = 64'h0;

		master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
	end

//
//	Begin multiple word write burst tests - unaligned +8
//
	for( index_4k = 9 ; index_4k < 16 ; index_4k++ ) begin
		command_addr = 8 + ( index_4k * ( 4 * 1024 ) ) + ( 16 * 1024 ) + `PCIE_64BIT_TEST_ADDR_OFFSET;
		command_addr_base = command_addr;

	//
	//	Write 2 thru 8 word bursts with varied byte enables
	//
		burst_count = index_4k - 7;
		burst_size = burst_count;
		idle = 0;  			// no idle cycle between each command of the master BFM
		init_latency = 0;  	// the command is launched to Avalon bus with no delay
		
		command_addr = command_addr - ( burst_count * 8 );
		command_addr_base = command_addr;

		byte_enable = 8'hFF;
		last_byte_enable = 8'hFF;
		for ( int i = 0 ; i < 64 ; i++ ) begin
		

			master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
			
			byte_enable = byte_enable << 1;
			last_byte_enable = last_byte_enable >> 1;
			
			if( byte_enable == 0 ) begin

				byte_enable = 8'hFF;
				last_byte_enable = 8'hFF;
			end
			
			command_addr = command_addr + ( burst_count * 8 );
		end

	//
	//	Read back and validate the write data to this point
	//
		idle = 0;  			// no idle cycle between each command of the master BFM
		init_latency = 0;  	// the command is launched to Avalon bus with no delay
		
		burst_count = ( command_addr - command_addr_base ) / 8;
		burst_size = burst_count;

		byte_enable = 8'hFF;
		command_data = 64'h0;

		master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
	end

//
//	Begin multiple word write burst tests - unaligned -8
//
	for( index_4k = 16 ; index_4k < 23 ; index_4k++ ) begin
		command_addr = ( ( ( index_4k + 1 ) * ( 4 * 1024 ) ) - 8 ) + ( 16 * 1024 ) + `PCIE_64BIT_TEST_ADDR_OFFSET;
		command_addr_base = command_addr;

	//
	//	Write 2 thru 8 word bursts with varied byte enables
	//
		burst_count = index_4k - 14;
		burst_size = burst_count;
		idle = 0;  			// no idle cycle between each command of the master BFM
		init_latency = 0;  	// the command is launched to Avalon bus with no delay
		
		byte_enable = 8'hFF;
		last_byte_enable = 8'hFF;
		for ( int i = 0 ; i < 64 ; i++ ) begin
		

			master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
			
			byte_enable = byte_enable << 1;
			last_byte_enable = last_byte_enable >> 1;
			
			if( byte_enable == 0 ) begin

				byte_enable = 8'hFF;
				last_byte_enable = 8'hFF;
			end
			
			command_addr = command_addr + ( burst_count * 8 );
		end

	//
	//	Read back and validate the write data to this point
	//
		idle = 0;  			// no idle cycle between each command of the master BFM
		init_latency = 0;  	// the command is launched to Avalon bus with no delay
		
		burst_count = ( command_addr - command_addr_base ) / 8;
		burst_size = burst_count;

		byte_enable = 8'hFF;
		command_data = 64'h0;

		master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
	end
	
//
//	Begin large multiple word write burst tests - aligned
//
	index_4k = 24; // use this for next write block

	//
	//	Write 2 thru 512 word bursts with varied byte enables
	//
	burst_count = 2;
	burst_size = burst_count;
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay

	byte_enable = 8'hFF;
	last_byte_enable = 8'hFF;
	
	command_data = 64'hFFFF_FFFF_FFFF_FFFF;
	
	while( burst_count > 0 ) begin
	
		command_addr = ( index_4k * ( 4 * 1024 ) ) + ( 16 * 1024 ) + `PCIE_64BIT_TEST_ADDR_OFFSET;
		command_addr_base = command_addr;

		master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );

	//
	//	Read back and validate the write data to this point
	//
		master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
		burst_count = burst_count << 1;
		burst_size = burst_count;
		index_4k++;
	end

//
//	Begin large multiple word write burst tests - unaligned +8
//
	index_4k = 34; // use this for next write block

	//
	//	Write 2 thru 512 word bursts with varied byte enables
	//
	burst_count = 2;
	burst_size = burst_count;
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay

	byte_enable = 8'hFF;
	last_byte_enable = 8'hFF;
	
	command_data = 64'hFFFF_FFFF_FFFF_FFFF;
	
	while( burst_count > 0 ) begin
	
		command_addr = 8 + ( index_4k * ( 4 * 1024 ) ) + ( 16 * 1024 ) + `PCIE_64BIT_TEST_ADDR_OFFSET;
		command_addr_base = command_addr - ( burst_count * 8 );
		
		master_set_and_push_command_write_burst( command_addr_base, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
			

	//
	//	Read back and validate the write data to this point
	//
		master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
		burst_count = burst_count << 1;
		burst_size = burst_count;
		index_4k++;
	end

//
//	Begin large multiple word write burst tests - unaligned -8
//
	index_4k = 44; // use this for next write block

	//
	//	Write 2 thru 512 word bursts with varied byte enables
	//
	burst_count = 2;
	burst_size = burst_count;
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay

	byte_enable = 8'hFF;
	last_byte_enable = 8'hFF;
	
	command_data = 64'hFFFF_FFFF_FFFF_FFFF;
	
	while( burst_count > 0 ) begin
	
		command_addr = ( ( ( index_4k + 1 ) * ( 4 * 1024 ) ) - 8 ) + ( 16 * 1024 ) + `PCIE_64BIT_TEST_ADDR_OFFSET;
		command_addr_base = command_addr;

		master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
			

	//
	//	Read back and validate the write data to this point
	//
		master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
		burst_count = burst_count << 1;
		burst_size = burst_count;
		index_4k++;
	end


////
//// do 4 individual dw writes to PCIe
////
//	command_addr = 0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0F;
//	command_data = 'h123456789abcdef0;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//	byte_enable = 'hF0;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 8 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0F;
//	command_data = 'hfedcba9876543210;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//	byte_enable = 'hF0;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read back the 4 individual dw from PCIe
////
//	command_addr = 0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0F;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//	byte_enable = 'hF0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 8 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0F;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//	byte_enable = 'hF0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// do 2 1 word avalon burst writes to PCIe
////
//	command_addr = 'h10 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	command_data = 'hfedcba9876543210;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h18 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	command_data = 'h123456789abcdef0;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read back the 2 1 word avalon bursts from PCIe
////
//	command_addr = 'h10 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h18 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write individual bytes to PCIe
////
//	command_addr = 'h100 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h01;
//	command_data = 'h1716151413121110;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h108 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h02;
//	command_data = 'h1716151413121110;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h110 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h04;
//	command_data = 'h1716151413121110;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h118 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h08;
//	command_data = 'h1716151413121110;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h120 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h10;
//	command_data = 'h1716151413121110;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h128 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h20;
//	command_data = 'h1716151413121110;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h130 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h40;
//	command_data = 'h1716151413121110;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h138 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h80;
//	command_data = 'h1716151413121110;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read individual bytes from PCIe
////
//	command_addr = 'h100 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h01;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h108 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h02;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h110 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h04;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h118 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h08;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h120 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h10;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h128 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h20;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h130 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h40;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h138 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h80;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write individual 2-byte to PCIe
////
//	command_addr = 'h140 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h03;
//	command_data = 'h2726252423222120;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h148 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h06;
//	command_data = 'h2726252423222120;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h150 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0C;
//	command_data = 'h2726252423222120;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h158 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h18;
//	command_data = 'h2726252423222120;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h160 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h30;
//	command_data = 'h2726252423222120;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h168 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h60;
//	command_data = 'h2726252423222120;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h170 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hC0;
//	command_data = 'h2726252423222120;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read individual 2-byte from PCIe
////
//	command_addr = 'h140 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h03;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h148 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h06;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h150 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0C;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h158 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h18;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h160 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h30;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h168 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h60;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h170 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hC0;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write individual 3-byte to PCIe
////
//	command_addr = 'h180 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h07;
//	command_data = 'h3736353433323130;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h188 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0E;
//	command_data = 'h3736353433323130;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h190 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h1C;
//	command_data = 'h3736353433323130;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h198 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h38;
//	command_data = 'h3736353433323130;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1A0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h70;
//	command_data = 'h3736353433323130;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1A8 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hE0;
//	command_data = 'h3736353433323130;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read individual 3-byte from PCIe
////
//	command_addr = 'h180 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h07;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h188 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0E;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h190 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h1C;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h198 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h38;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1A0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h70;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1A8 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hE0;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write individual 4-byte to PCIe
////
//	command_addr = 'h1B0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0F;
//	command_data = 'h4746454443424140;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1B8 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h1E;
//	command_data = 'h4746454443424140;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1C0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h3C;
//	command_data = 'h4746454443424140;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1C8 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h78;
//	command_data = 'h4746454443424140;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1D0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF0;
//	command_data = 'h4746454443424140;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read individual 4-byte from PCIe
////
//	command_addr = 'h1B0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0F;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1B8 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h1E;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1C0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h3C;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1C8 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h78;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1D0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF0;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write individual 5-byte to PCIe
////
//	command_addr = 'h1E0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h1F;
//	command_data = 'h5756555453525150;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1E8 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h3E;
//	command_data = 'h5756555453525150;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1F0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h7C;
//	command_data = 'h5756555453525150;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1F8 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF8;
//	command_data = 'h5756555453525150;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read individual 5-byte to PCIe
////
//	command_addr = 'h1E0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h1F;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1E8 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h3E;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1F0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h7C;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1F8 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF8;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write individual 6-byte to PCIe
////
//	command_addr = 'h200 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h3F;
//	command_data = 'h6766656463626160;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h208 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h7E;
//	command_data = 'h6766656463626160;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h210 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFC;
//	command_data = 'h6766656463626160;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read individual 6-byte from PCIe
////
//	command_addr = 'h200 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h3F;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h208 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h7E;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h210 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFC;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write individual 7-byte to PCIe
////
//	command_addr = 'h220 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h7F;
//	command_data = 'h7776757473727170;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h228 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFE;
//	command_data = 'h7776757473727170;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read individual 7-byte to PCIe
////
//	command_addr = 'h220 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h7F;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h228 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFE;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write individual 8-byte to PCIe
////
//	command_addr = 'h230 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	command_data = 'h8786858483828180;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read individual 8-byte from PCIe
////
//	command_addr = 'h230 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read 512-byte block from PCIe
////
//	command_addr = 'h100 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, 64, 64, idle, init_latency);
//
//	wait( checked == request_count );

////
//// write 2 Avalon burst block to PCIe
////
//	burst_count = 2;
//	burst_size = 2;
//
//	command_addr = 'h240 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	last_byte_enable = 'hFF;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h250 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFE;
//	last_byte_enable = 'h7F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h260 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFC;
//	last_byte_enable = 'h3F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h270 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF8;
//	last_byte_enable = 'h1F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h280 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF0;
//	last_byte_enable = 'h0F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h290 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hE0;
//	last_byte_enable = 'h07;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h2A0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hC0;
//	last_byte_enable = 'h03;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h2B0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h80;
//	last_byte_enable = 'h01;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
////
//// read 2 Avalon burst block from PCIe
////
//	burst_count = 2;
//	burst_size = 2;
//	byte_enable = 'hFF;
//
//	command_addr = 'h240 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h250 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h260 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h270 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h280 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h290 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h2A0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h2B0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write 3 Avalon burst block to PCIe
////
//	burst_count = 3;
//	burst_size = 3;
//
//	command_addr = 'h2C0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	last_byte_enable = 'hFF;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h300 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFE;
//	last_byte_enable = 'h7F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h340 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFC;
//	last_byte_enable = 'h3F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h380 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF8;
//	last_byte_enable = 'h1F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h3C0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF0;
//	last_byte_enable = 'h0F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h400 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hE0;
//	last_byte_enable = 'h07;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h440 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hC0;
//	last_byte_enable = 'h03;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h480 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h80;
//	last_byte_enable = 'h01;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
////
//// read 3 Avalon burst block from PCIe
////
//	burst_count = 3;
//	burst_size = 3;
//	byte_enable = 'hFF;
//
//	command_addr = 'h2C0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h300 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h340 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h380 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h3C0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h400 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h440 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h480 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write 4 Avalon burst block to PCIe
////
//	burst_count = 4;
//	burst_size = 4;
//
//	command_addr = 'h4C0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	last_byte_enable = 'hFF;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h500 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFE;
//	last_byte_enable = 'h7F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h540 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFC;
//	last_byte_enable = 'h3F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h580 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF8;
//	last_byte_enable = 'h1F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h5C0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF0;
//	last_byte_enable = 'h0F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h600 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hE0;
//	last_byte_enable = 'h07;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h640 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hC0;
//	last_byte_enable = 'h03;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h680 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h80;
//	last_byte_enable = 'h01;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
////
//// read 4 Avalon burst block from PCIe
////
//	burst_count = 4;
//	burst_size = 4;
//	byte_enable = 'hFF;
//
//	command_addr = 'h4C0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h500 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h540 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h580 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h5C0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h600 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h640 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h680 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write 5 thru 16 Avalon burst block to PCIe
////
//	command_addr = 'h700 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	burst_count = 5;
//	burst_size = 5;
//	byte_enable = 'hFF;
//	last_byte_enable = 'hFF;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h740 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	burst_count = 6;
//	burst_size = 6;
//	byte_enable = 'hFE;
//	last_byte_enable = 'h7F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h780 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	burst_count = 7;
//	burst_size = 7;
//	byte_enable = 'hFC;
//	last_byte_enable = 'h3F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h7C0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	burst_count = 8;
//	burst_size = 8;
//	byte_enable = 'hF8;
//	last_byte_enable = 'h1F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h800 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	burst_count = 9;
//	burst_size = 9;
//	byte_enable = 'hF0;
//	last_byte_enable = 'h0F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h880 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	burst_count = 10;
//	burst_size = 10;
//	byte_enable = 'hE0;
//	last_byte_enable = 'h07;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h900 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	burst_count = 11;
//	burst_size = 11;
//	byte_enable = 'hC0;
//	last_byte_enable = 'h03;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h980 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	burst_count = 12;
//	burst_size = 12;
//	byte_enable = 'h80;
//	last_byte_enable = 'h01;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'hA00 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	burst_count = 13;
//	burst_size = 13;
//	byte_enable = 'hFF;
//	last_byte_enable = 'hFF;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'hA80 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	burst_count = 14;
//	burst_size = 14;
//	byte_enable = 'hFE;
//	last_byte_enable = 'h7F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'hB00 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	burst_count = 15;
//	burst_size = 15;
//	byte_enable = 'hFC;
//	last_byte_enable = 'h3F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'hB80 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	burst_count = 16;
//	burst_size = 16;
//	byte_enable = 'hF8;
//	last_byte_enable = 'h1F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
////
//// read 5 thru 16 Avalon burst block from PCIe
////
//	byte_enable = 'hFF;
//
//	burst_count = 5;
//	burst_size = 5;
//	command_addr = 'h700 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 6;
//	burst_size = 6;
//	command_addr = 'h740 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 7;
//	burst_size = 7;
//	command_addr = 'h780 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 8;
//	burst_size = 8;
//	command_addr = 'h7C0 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 9;
//	burst_size = 9;
//	command_addr = 'h800 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 10;
//	burst_size = 10;
//	command_addr = 'h880 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 11;
//	burst_size = 11;
//	command_addr = 'h900 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 12;
//	burst_size = 12;
//	command_addr = 'h980 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 13;
//	burst_size = 13;
//	command_addr = 'hA00 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 14;
//	burst_size = 14;
//	command_addr = 'hA80 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 15;
//	burst_size = 15;
//	command_addr = 'hB00 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 16;
//	burst_size = 16;
//	command_addr = 'hB80 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
////	write sensitive burst overlaps
////
//	burst_count = 16;
//	burst_size = 16;
//	byte_enable = 'hFF;
//	last_byte_enable = 'hFF;
//
//	command_addr = 'hC08 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'hCF8 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'hD90 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'hE70 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	burst_count = 17;
//	burst_size = 17;
//	command_addr = 'hEF8 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'hF80 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	burst_count = 18;
//	burst_size = 18;
//	command_addr = 'h1078 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	burst_count = 34;
//	burst_size = 34;
//	command_addr = 'h1178 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h1F78 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
////
////	read sensitive burst overlaps
////
//	byte_enable = 'hFF;
//	burst_count = 16;
//	burst_size = 16;
//
//	command_addr = 'hC08 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'hCF8 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'hD90 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'hE70 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 17;
//	burst_size = 17;
//	command_addr = 'hEF8 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'hF80 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 18;
//	burst_size = 18;
//	command_addr = 'h1078 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 34;
//	burst_size = 34;
//	command_addr = 'h1178 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1F78 + `PCIE_32BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);


////
//// TOGGLE to 64-bit address
////
//	wait( checked == request_count );
//
//	burst_count = 1;		// signal burst 1 word
//	burst_size = 1;		// and burst 1 word
//	idle = 0;  			// no idle cycle between each command of the master BFM
//	init_latency = 0;  	// the command is launched to Avalon bus with no delay
//
//	command_addr = `SLAVE_0_CSR_BASE;
//	byte_enable = 'hFF;
//	command_data = 'h0000000100000000;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// do 4 individual dw writes to PCIe
////
//	command_addr = 0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0F;
//	command_data = 'h123456789abcdef0;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//	byte_enable = 'hF0;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 8 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0F;
//	command_data = 'hfedcba9876543210;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//	byte_enable = 'hF0;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read back the 4 individual dw from PCIe
////
//	command_addr = 0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0F;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//	byte_enable = 'hF0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 8 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0F;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//	byte_enable = 'hF0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// do 2 1 word avalon burst writes to PCIe
////
//	command_addr = 'h10 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	command_data = 'hfedcba9876543210;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h18 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	command_data = 'h123456789abcdef0;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read back the 2 1 word avalon bursts from PCIe
////
//	command_addr = 'h10 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h18 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write individual bytes to PCIe
////
//	command_addr = 'h100 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h01;
//	command_data = 'h1716151413121110;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h108 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h02;
//	command_data = 'h1716151413121110;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h110 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h04;
//	command_data = 'h1716151413121110;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h118 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h08;
//	command_data = 'h1716151413121110;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h120 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h10;
//	command_data = 'h1716151413121110;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h128 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h20;
//	command_data = 'h1716151413121110;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h130 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h40;
//	command_data = 'h1716151413121110;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h138 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h80;
//	command_data = 'h1716151413121110;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read individual bytes from PCIe
////
//	command_addr = 'h100 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h01;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h108 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h02;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h110 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h04;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h118 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h08;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h120 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h10;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h128 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h20;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h130 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h40;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h138 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h80;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write individual 2-byte to PCIe
////
//	command_addr = 'h140 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h03;
//	command_data = 'h2726252423222120;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h148 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h06;
//	command_data = 'h2726252423222120;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h150 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0C;
//	command_data = 'h2726252423222120;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h158 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h18;
//	command_data = 'h2726252423222120;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h160 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h30;
//	command_data = 'h2726252423222120;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h168 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h60;
//	command_data = 'h2726252423222120;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h170 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hC0;
//	command_data = 'h2726252423222120;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read individual 2-byte from PCIe
////
//	command_addr = 'h140 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h03;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h148 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h06;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h150 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0C;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h158 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h18;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h160 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h30;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h168 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h60;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h170 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hC0;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write individual 3-byte to PCIe
////
//	command_addr = 'h180 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h07;
//	command_data = 'h3736353433323130;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h188 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0E;
//	command_data = 'h3736353433323130;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h190 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h1C;
//	command_data = 'h3736353433323130;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h198 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h38;
//	command_data = 'h3736353433323130;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1A0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h70;
//	command_data = 'h3736353433323130;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1A8 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hE0;
//	command_data = 'h3736353433323130;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read individual 3-byte from PCIe
////
//	command_addr = 'h180 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h07;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h188 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0E;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h190 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h1C;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h198 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h38;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1A0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h70;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1A8 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hE0;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write individual 4-byte to PCIe
////
//	command_addr = 'h1B0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0F;
//	command_data = 'h4746454443424140;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1B8 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h1E;
//	command_data = 'h4746454443424140;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1C0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h3C;
//	command_data = 'h4746454443424140;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1C8 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h78;
//	command_data = 'h4746454443424140;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1D0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF0;
//	command_data = 'h4746454443424140;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read individual 4-byte from PCIe
////
//	command_addr = 'h1B0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h0F;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1B8 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h1E;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1C0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h3C;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1C8 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h78;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1D0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF0;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write individual 5-byte to PCIe
////
//	command_addr = 'h1E0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h1F;
//	command_data = 'h5756555453525150;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1E8 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h3E;
//	command_data = 'h5756555453525150;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1F0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h7C;
//	command_data = 'h5756555453525150;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1F8 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF8;
//	command_data = 'h5756555453525150;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read individual 5-byte to PCIe
////
//	command_addr = 'h1E0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h1F;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1E8 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h3E;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1F0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h7C;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1F8 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF8;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write individual 6-byte to PCIe
////
//	command_addr = 'h200 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h3F;
//	command_data = 'h6766656463626160;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h208 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h7E;
//	command_data = 'h6766656463626160;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h210 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFC;
//	command_data = 'h6766656463626160;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read individual 6-byte from PCIe
////
//	command_addr = 'h200 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h3F;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h208 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h7E;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h210 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFC;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write individual 7-byte to PCIe
////
//	command_addr = 'h220 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h7F;
//	command_data = 'h7776757473727170;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h228 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFE;
//	command_data = 'h7776757473727170;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read individual 7-byte to PCIe
////
//	command_addr = 'h220 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h7F;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h228 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFE;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write individual 8-byte to PCIe
////
//	command_addr = 'h230 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	command_data = 'h8786858483828180;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read individual 8-byte from PCIe
////
//	command_addr = 'h230 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// read 512-byte block from PCIe
////
//	command_addr = 'h100 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, 64, 64, idle, init_latency);
//
//	wait( checked == request_count );

////
//// write 2 Avalon burst block to PCIe
////
//	burst_count = 2;
//	burst_size = 2;
//
//	command_addr = 'h240 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	last_byte_enable = 'hFF;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h250 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFE;
//	last_byte_enable = 'h7F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h260 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFC;
//	last_byte_enable = 'h3F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h270 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF8;
//	last_byte_enable = 'h1F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h280 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF0;
//	last_byte_enable = 'h0F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h290 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hE0;
//	last_byte_enable = 'h07;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h2A0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hC0;
//	last_byte_enable = 'h03;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h2B0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h80;
//	last_byte_enable = 'h01;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
////
//// read 2 Avalon burst block from PCIe
////
//	burst_count = 2;
//	burst_size = 2;
//	byte_enable = 'hFF;
//
//	command_addr = 'h240 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h250 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h260 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h270 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h280 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h290 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h2A0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h2B0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write 3 Avalon burst block to PCIe
////
//	burst_count = 3;
//	burst_size = 3;
//
//	command_addr = 'h2C0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	last_byte_enable = 'hFF;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h300 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFE;
//	last_byte_enable = 'h7F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h340 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFC;
//	last_byte_enable = 'h3F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h380 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF8;
//	last_byte_enable = 'h1F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h3C0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF0;
//	last_byte_enable = 'h0F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h400 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hE0;
//	last_byte_enable = 'h07;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h440 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hC0;
//	last_byte_enable = 'h03;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h480 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h80;
//	last_byte_enable = 'h01;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
////
//// read 3 Avalon burst block from PCIe
////
//	burst_count = 3;
//	burst_size = 3;
//	byte_enable = 'hFF;
//
//	command_addr = 'h2C0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h300 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h340 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h380 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h3C0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h400 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h440 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h480 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write 4 Avalon burst block to PCIe
////
//	burst_count = 4;
//	burst_size = 4;
//
//	command_addr = 'h4C0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFF;
//	last_byte_enable = 'hFF;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h500 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFE;
//	last_byte_enable = 'h7F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h540 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hFC;
//	last_byte_enable = 'h3F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h580 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF8;
//	last_byte_enable = 'h1F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h5C0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hF0;
//	last_byte_enable = 'h0F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h600 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hE0;
//	last_byte_enable = 'h07;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h640 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'hC0;
//	last_byte_enable = 'h03;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h680 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	byte_enable = 'h80;
//	last_byte_enable = 'h01;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
////
//// read 4 Avalon burst block from PCIe
////
//	burst_count = 4;
//	burst_size = 4;
//	byte_enable = 'hFF;
//
//	command_addr = 'h4C0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h500 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h540 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h580 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h5C0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h600 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h640 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h680 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
//// write 5 thru 16 Avalon burst block to PCIe
////
//	command_addr = 'h700 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	burst_count = 5;
//	burst_size = 5;
//	byte_enable = 'hFF;
//	last_byte_enable = 'hFF;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h740 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	burst_count = 6;
//	burst_size = 6;
//	byte_enable = 'hFE;
//	last_byte_enable = 'h7F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h780 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	burst_count = 7;
//	burst_size = 7;
//	byte_enable = 'hFC;
//	last_byte_enable = 'h3F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h7C0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	burst_count = 8;
//	burst_size = 8;
//	byte_enable = 'hF8;
//	last_byte_enable = 'h1F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h800 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	burst_count = 9;
//	burst_size = 9;
//	byte_enable = 'hF0;
//	last_byte_enable = 'h0F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h880 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	burst_count = 10;
//	burst_size = 10;
//	byte_enable = 'hE0;
//	last_byte_enable = 'h07;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h900 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	burst_count = 11;
//	burst_size = 11;
//	byte_enable = 'hC0;
//	last_byte_enable = 'h03;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h980 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	burst_count = 12;
//	burst_size = 12;
//	byte_enable = 'h80;
//	last_byte_enable = 'h01;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'hA00 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	burst_count = 13;
//	burst_size = 13;
//	byte_enable = 'hFF;
//	last_byte_enable = 'hFF;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'hA80 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	burst_count = 14;
//	burst_size = 14;
//	byte_enable = 'hFE;
//	last_byte_enable = 'h7F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'hB00 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	burst_count = 15;
//	burst_size = 15;
//	byte_enable = 'hFC;
//	last_byte_enable = 'h3F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'hB80 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	burst_count = 16;
//	burst_size = 16;
//	byte_enable = 'hF8;
//	last_byte_enable = 'h1F;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
////
//// read 5 thru 16 Avalon burst block from PCIe
////
//	byte_enable = 'hFF;
//
//	burst_count = 5;
//	burst_size = 5;
//	command_addr = 'h700 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 6;
//	burst_size = 6;
//	command_addr = 'h740 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 7;
//	burst_size = 7;
//	command_addr = 'h780 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 8;
//	burst_size = 8;
//	command_addr = 'h7C0 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 9;
//	burst_size = 9;
//	command_addr = 'h800 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 10;
//	burst_size = 10;
//	command_addr = 'h880 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 11;
//	burst_size = 11;
//	command_addr = 'h900 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 12;
//	burst_size = 12;
//	command_addr = 'h980 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 13;
//	burst_size = 13;
//	command_addr = 'hA00 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 14;
//	burst_size = 14;
//	command_addr = 'hA80 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 15;
//	burst_size = 15;
//	command_addr = 'hB00 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 16;
//	burst_size = 16;
//	command_addr = 'hB80 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
////
////	write sensitive burst overlaps
////
//	burst_count = 16;
//	burst_size = 16;
//	byte_enable = 'hFF;
//	last_byte_enable = 'hFF;
//
//	command_addr = 'hC08 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'hCF8 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'hD90 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'hE70 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	burst_count = 17;
//	burst_size = 17;
//	command_addr = 'hEF8 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'hF80 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	burst_count = 18;
//	burst_size = 18;
//	command_addr = 'h1078 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	burst_count = 34;
//	burst_size = 34;
//	command_addr = 'h1178 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
//	command_addr = 'h1F78 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
//
////
////	read sensitive burst overlaps
////
//	byte_enable = 'hFF;
//	burst_count = 16;
//	burst_size = 16;
//
//	command_addr = 'hC08 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'hCF8 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'hD90 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'hE70 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 17;
//	burst_size = 17;
//	command_addr = 'hEF8 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'hF80 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 18;
//	burst_size = 18;
//	command_addr = 'h1078 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	burst_count = 34;
//	burst_size = 34;
//	command_addr = 'h1178 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//	command_addr = 'h1F78 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);

////
//// TOGGLE back to 32-bit address
////
//	wait( checked == request_count );
//
//	burst_count = 1;		// signal burst 1 word
//	burst_size = 1;		// and burst 1 word
//	idle = 0;  			// no idle cycle between each command of the master BFM
//	init_latency = 0;  	// the command is launched to Avalon bus with no delay
//
//	command_addr = `SLAVE_0_CSR_BASE;
//	byte_enable = 'hFF;
//	command_data = 'h0000000000000000;
//	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//	command_data = 'h0;
//	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);

////
////	read sensitive burst overlaps
////
//	byte_enable = 'hFF;
//	burst_count = 24;
//	burst_size = 24;
//
//	for (int i = 0; i < 20; i++) 
//	begin
//		command_addr = 'h000 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//		master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//		command_addr = 'h200 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//		master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//		command_addr = 'h400 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//		master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//
//		command_addr = 'h600 + `PCIE_64BIT_TEST_ADDR_OFFSET;
//		master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
//	end   

//
// Signal to the test bench that we've reached the end of the input.
//
	input_complete = 1;

end
