//change this file for different test case
initial
begin
  @ start_test;
  
//
//	Read all valid byte combinations thru an entire 64-bit word
//
	burst_count = 1;	// signal burst 1 word
	burst_size = 1;		// and burst 1 word
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	command_addr = `PCIE_32BIT_TEST_ADDR_OFFSET;
	command_data = 64'h0123_4567_89AB_CDEF;

	for ( int j = 0 ; j < 8 ; j++ ) begin

		byte_enable = 8'hFF >> ( 7 - j );
		command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF >> ( ( 7 - j ) * 8 );
		for ( int i = 0 ; i < ( 8 - j ) ; i++ ) begin
		
			master_data_scoreboard.push_back( command_data );
			master_mask_scoreboard.push_back( command_data_mask );
			
			master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
			
			command_data_mask = command_data_mask << 8;
			byte_enable = byte_enable << 1;
		end
	end

//
//	Read 2 64-bit word burst
//
	burst_count = 2;	// signal burst 2 word
	burst_size = 2;		// and burst 2 word
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	
	command_addr = `PCIE_32BIT_TEST_ADDR_OFFSET;
	byte_enable = 8'hFF;

	command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF;	
	command_data = 64'h0123_4567_89AB_CDEF;
	master_data_scoreboard.push_back( command_data );
	master_mask_scoreboard.push_back( command_data_mask );

	command_data = 64'hFEDC_BA98_7654_3210;
	master_data_scoreboard.push_back( command_data );
	master_mask_scoreboard.push_back( command_data_mask );

	command_data = 64'h0;
	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);

//
//	Read aligned large burst blocks from 16 byte thru 4096 byte
//
	command_addr = ( 4 * 1024 ) + `PCIE_32BIT_TEST_ADDR_OFFSET;
	byte_enable = 8'hFF;
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	
	burst_count = ( 16 / 8 );
	burst_size = burst_count;

	command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF;
	
	while( burst_count > 0 ) begin
		command_data = 64'h0;
		for ( int i = 0 ; i < burst_count ; i++ ) begin
			master_data_scoreboard.push_back( command_data );
			master_mask_scoreboard.push_back( command_data_mask );
			command_data = command_data + 1;
		end

		command_data = 64'h0;
		master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
		burst_count = burst_count << 1;
		burst_size = burst_count;
	end

//
//	Begin single word write tests
//
	index_4k = 0;
	command_addr = ( index_4k * ( 4 * 1024 ) ) + ( 16 * 1024 ) + `PCIE_32BIT_TEST_ADDR_OFFSET;
	command_addr_base = command_addr;

//
//	Write all valid byte combinations thru an entire 64-bit word
//
	burst_count = 1;	// signal burst 1 word
	burst_size = 1;		// and burst 1 word
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	command_data = 64'h8786858483828180;
	
	for( int j = 0 ; j < 8 ; j++ ) begin

		byte_enable = 8'hFF >> j;
		command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF >> ( j * 8 );
		for ( int i = 0 ; i < ( j + 1 ) ; i++ ) begin
		
			master_data_scoreboard.push_back( command_data );
			master_mask_scoreboard.push_back( command_data_mask );
			
			master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
			
			command_data_mask = command_data_mask << 8;
			byte_enable = byte_enable << 1;
			
			command_addr = command_addr + 8;
		end
		command_data = command_data - 64'h1010_1010_1010_1010;
	end
//
//	Read back and validate the write data to this point
//
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	
	burst_count = ( command_addr - command_addr_base ) / 8;
	burst_size = burst_count;

	byte_enable = 8'hFF;
	command_data = 64'h0;

	master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
//
//	Begin large multiple word write burst tests - aligned
//
	index_4k = 24; // use this for next write block

	//
	//	Write 2 thru 512 word bursts with varied byte enables
	//
	burst_count = 2;
	burst_size = burst_count;
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay

	byte_enable = 8'hFF;
	last_byte_enable = 8'hFF;
	
	command_data = 64'hFFFF_FFFF_FFFF_FFFF;
	
	while( burst_count > 0 ) begin
	
		command_addr = ( index_4k * ( 4 * 1024 ) ) + ( 16 * 1024 ) + `PCIE_32BIT_TEST_ADDR_OFFSET;
		command_addr_base = command_addr;

		master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );

	//
	//	Read back and validate the write data to this point
	//
		master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
		burst_count = burst_count << 1;
		burst_size = burst_count;
		index_4k++;
	end

//
// TOGGLE to 64-bit address
//
	wait( checked == request_count );

	burst_count = 1;		// signal burst 1 word
	burst_size = 1;		// and burst 1 word
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay

	command_addr = `SLAVE_0_CSR_BASE;
	byte_enable = 'hFF;
	command_data = 'h0000000100000000;
	master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);

//
//	Read all valid byte combinations thru an entire 64-bit word
//
	burst_count = 1;	// signal burst 1 word
	burst_size = 1;		// and burst 1 word
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	command_addr = `PCIE_64BIT_TEST_ADDR_OFFSET;
	command_data = 64'hFEDC_BA98_7654_3210;

	for ( int j = 0 ; j < 8 ; j++ ) begin

		byte_enable = 8'hFF >> ( 7 - j );
		command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF >> ( ( 7 - j ) * 8 );
		for ( int i = 0 ; i < ( 8 - j ) ; i++ ) begin
		
			master_data_scoreboard.push_back( command_data );
			master_mask_scoreboard.push_back( command_data_mask );
			
			master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
			
			command_data_mask = command_data_mask << 8;
			byte_enable = byte_enable << 1;
		end
	end

//
//	Read 2 64-bit word burst
//
	burst_count = 2;	// signal burst 2 word
	burst_size = 2;		// and burst 2 word
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	
	command_addr = `PCIE_64BIT_TEST_ADDR_OFFSET;
	byte_enable = 8'hFF;

	command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF;	
	command_data = 64'hFEDC_BA98_7654_3210;
	master_data_scoreboard.push_back( command_data );
	master_mask_scoreboard.push_back( command_data_mask );

	command_data = 64'h0123_4567_89AB_CDEF;
	master_data_scoreboard.push_back( command_data );
	master_mask_scoreboard.push_back( command_data_mask );

	command_data = 64'h0;
	master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);

//
//	Read aligned large burst blocks from 16 byte thru 4096 byte
//
	command_addr = ( 4 * 1024 ) + `PCIE_64BIT_TEST_ADDR_OFFSET;
	byte_enable = 8'hFF;
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	
	burst_count = ( 16 / 8 );
	burst_size = burst_count;

	command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF;
	
	while( burst_count > 0 ) begin
		command_data = 64'h0;
		for ( int i = 0 ; i < burst_count ; i++ ) begin
			master_data_scoreboard.push_back( command_data );
			master_mask_scoreboard.push_back( command_data_mask );
			command_data = command_data + 1;
		end

		command_data = 64'h0;
		master_set_and_push_command(REQ_READ, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
		burst_count = burst_count << 1;
		burst_size = burst_count;
	end

//
//	Begin single word write tests
//
	index_4k = 0;
	command_addr = ( index_4k * ( 4 * 1024 ) ) + ( 16 * 1024 ) + `PCIE_64BIT_TEST_ADDR_OFFSET;
	command_addr_base = command_addr;

//
//	Write all valid byte combinations thru an entire 64-bit word
//
	burst_count = 1;	// signal burst 1 word
	burst_size = 1;		// and burst 1 word
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	command_data = 64'h8786858483828180;
	
	for( int j = 0 ; j < 8 ; j++ ) begin

		byte_enable = 8'hFF >> j;
		command_data_mask = 64'hFFFF_FFFF_FFFF_FFFF >> ( j * 8 );
		for ( int i = 0 ; i < ( j + 1 ) ; i++ ) begin
		
			master_data_scoreboard.push_back( command_data );
			master_mask_scoreboard.push_back( command_data_mask );
			
			master_set_and_push_command(REQ_WRITE, command_addr, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
			
			command_data_mask = command_data_mask << 8;
			byte_enable = byte_enable << 1;
			
			command_addr = command_addr + 8;
		end
		command_data = command_data - 64'h1010_1010_1010_1010;
	end
//
//	Read back and validate the write data to this point
//
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay
	
	burst_count = ( command_addr - command_addr_base ) / 8;
	burst_size = burst_count;

	byte_enable = 8'hFF;
	command_data = 64'h0;

	master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
//
//	Begin multiple word write burst tests - aligned
//
	for( index_4k = 1 ; index_4k < 8 ; index_4k++ ) begin
		command_addr = ( index_4k * ( 4 * 1024 ) ) + ( 16 * 1024 ) + `PCIE_64BIT_TEST_ADDR_OFFSET;
		command_addr_base = command_addr;

	//
	//	Write 2 thru 8 word bursts with varied byte enables
	//
		burst_count = index_4k + 1;
		burst_size = burst_count;
		idle = 0;  			// no idle cycle between each command of the master BFM
		init_latency = 0;  	// the command is launched to Avalon bus with no delay
		
		byte_enable = 8'hFF;
		last_byte_enable = 8'hFF;
		for ( int i = 0 ; i < 64 ; i++ ) begin
		

			master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );
			
			byte_enable = byte_enable << 1;
			last_byte_enable = last_byte_enable >> 1;
			
			if( byte_enable == 0 ) begin

				byte_enable = 8'hFF;
				last_byte_enable = 8'hFF;
			end
			
			command_addr = command_addr + ( burst_count * 8 );
		end

	//
	//	Read back and validate the write data to this point
	//
		idle = 0;  			// no idle cycle between each command of the master BFM
		init_latency = 0;  	// the command is launched to Avalon bus with no delay
		
		burst_count = ( command_addr - command_addr_base ) / 8;
		burst_size = burst_count;

		byte_enable = 8'hFF;
		command_data = 64'h0;

		master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
	end

//
//	Begin large multiple word write burst tests - aligned
//
	index_4k = 24; // use this for next write block

	//
	//	Write 2 thru 512 word bursts with varied byte enables
	//
	burst_count = 2;
	burst_size = burst_count;
	idle = 0;  			// no idle cycle between each command of the master BFM
	init_latency = 0;  	// the command is launched to Avalon bus with no delay

	byte_enable = 8'hFF;
	last_byte_enable = 8'hFF;
	
	command_data = 64'hFFFF_FFFF_FFFF_FFFF;
	
	while( burst_count > 0 ) begin
	
		command_addr = ( index_4k * ( 4 * 1024 ) ) + ( 16 * 1024 ) + `PCIE_64BIT_TEST_ADDR_OFFSET;
		command_addr_base = command_addr;

		master_set_and_push_command_write_burst( command_addr, byte_enable, last_byte_enable, burst_count, burst_size, idle, init_latency );

	//
	//	Read back and validate the write data to this point
	//
		master_set_and_push_command(REQ_READ, command_addr_base, command_data, byte_enable, burst_count, burst_size, idle, init_latency);
		
		burst_count = burst_count << 1;
		burst_size = burst_count;
		index_4k++;
	end

//
// Signal to the test bench that we've reached the end of the input.
//
	input_complete = 1;

end
