#include "system.h"
#include "alt_types.h"
#include "io.h"
#include "altera_avalon_fifo_regs.h"

//#include "sys/alt_stdio.h"

//
// Simulation Print Macros
//
#define  TIME_PRINT              (0 * 4)
#define  STOP_SUCCESS            (1 * 4)
#define  STOP_FAIL               (2 * 4)
#define  ENABLE_MODULE_PRINT     (3 * 4)
#define  DISABLE_MODULE_PRINT    (4 * 4)
#define  PRINT_HEX_RAW           (5 * 4)
#define  PRINT_HEX_TYPED         (6 * 4)
#define  PRINT_DEC_RAW           (7 * 4)
#define  PRINT_DEC_TYPED         (8 * 4)
#define  PRINT_CHAR_RAW          (9 * 4)
#define  PRINT_LINE_FEED         (10 * 4)
#define  ENABLE_PRINT_STORE      (11 * 4)
#define  DISABLE_PRINT_STORE     (12 * 4)
#define  DUMP_PRINT_STORE        (13 * 4)

void simPrintString(char *str) {

    while(*str) {
        IOWR_8DIRECT(SIMULATIONPRINT_0_BASE, PRINT_CHAR_RAW, *str++);
    }
}

void simPrintTime(void) {

    IOWR_32DIRECT(SIMULATIONPRINT_0_BASE, TIME_PRINT, 0x0);
}

//
// TLP Header Macros
//
#define HDR_LENGTH_10_MASK	( 0x03FF )
#define HDR_LENGTH_10_OFST	( 0 )
#define HDR_RSV0_2_MASK		( 0x03 )
#define HDR_RSV0_2_OFST		( 10 )
#define HDR_ATTR_2_MASK		( 0x03 )
#define HDR_ATTR_2_OFST		( 12 )
#define HDR_EP_1_MASK		( 0x01 )
#define HDR_EP_1_OFST		( 14 )
#define HDR_TD_1_MASK		( 0x01 )
#define HDR_TD_1_OFST		( 15 )
#define HDR_RSV1_4_MASK		( 0x0F )
#define HDR_RSV1_4_OFST		( 16 )
#define HDR_TC_3_MASK		( 0x07 )
#define HDR_TC_3_OFST		( 20 )
#define HDR_RSV2_1_MASK		( 0x01 )
#define HDR_RSV2_1_OFST		( 23 )
#define HDR_TYPE_5_MASK		( 0x01F )
#define HDR_TYPE_5_OFST		( 24 )
#define HDR_FMT_2_MASK		( 0x03 )
#define HDR_FMT_2_OFST		( 29 )
#define HDR_RSV3_1_MASK		( 0x01 )
#define HDR_RSV3_1_OFST		( 31 )

#define HDR_FDWBE_4_MASK	( 0x0F )
#define HDR_FDWBE_4_OFST	( 0 )
#define HDR_LDWBE_4_MASK	( 0x0F )
#define HDR_LDWBE_4_OFST	( 4 )
#define HDR_TAG_8_MASK		( 0x0FF )
#define HDR_TAG_8_OFST		( 8 )
#define HDR_REQID_16_MASK	( 0x0FFFF )
#define HDR_REQID_16_OFST	( 16 )

#define REQID_FUNC_3_MASK	( 0x07 )
#define REQID_FUNC_3_OFST	( 0 )
#define REQID_DEV_5_MASK	( 0x01F )
#define REQID_DEV_5_OFST	( 3 )
#define REQID_BUS_8_MASK	( 0x0FF )
#define REQID_BUS_8_OFST	( 8 )

#define HDR_LENGTH_10(x)	( ( x & HDR_LENGTH_10_MASK )	<< HDR_LENGTH_10_OFST )
#define HDR_RSV0_2(x)		( ( x & HDR_RSV0_2_MASK ) 		<< HDR_RSV0_2_OFST )
#define HDR_ATTR_2(x)		( ( x & HDR_ATTR_2_MASK ) 		<< HDR_ATTR_2_OFST )
#define HDR_EP_1(x)			( ( x & HDR_EP_1_MASK ) 		<< HDR_EP_1_OFST )
#define HDR_TD_1(x)			( ( x & HDR_TD_1_MASK ) 		<< HDR_TD_1_OFST )
#define HDR_RSV1_4(x)		( ( x & HDR_RSV1_4_MASK )		<< HDR_RSV1_4_OFST )
#define HDR_TC_3(x)			( ( x & HDR_TC_3_MASK )			<< HDR_TC_3_OFST )
#define HDR_RSV2_1(x)		( ( x & HDR_RSV2_1_MASK )		<< HDR_RSV2_1_OFST )
#define HDR_TYPE_5(x)		( ( x & HDR_TYPE_5_MASK )		<< HDR_TYPE_5_OFST )
#define HDR_FMT_2(x)		( ( x & HDR_FMT_2_MASK ) 		<< HDR_FMT_2_OFST )
#define HDR_RSV3_1(x)		( ( x & HDR_RSV3_1_MASK ) 		<< HDR_RSV3_1_OFST )

#define HDR_FDWBE_4(x)		( ( x & HDR_FDWBE_4_MASK ) 		<< HDR_FDWBE_4_OFST )
#define HDR_LDWBE_4(x)		( ( x & HDR_LDWBE_4_MASK ) 		<< HDR_LDWBE_4_OFST )
#define HDR_TAG_8(x)		( ( x & HDR_TAG_8_MASK ) 		<< HDR_TAG_8_OFST )
#define HDR_REQID_16(x)		( ( x & HDR_REQID_16_MASK ) 	<< HDR_REQID_16_OFST )

#define REQID_FUNC_3(x)		( ( x & REQID_FUNC_3_MASK ) 	<< REQID_FUNC_3_OFST )
#define REQID_DEV_5(x)		( ( x & REQID_DEV_5_MASK ) 		<< REQID_DEV_5_OFST )
#define REQID_BUS_8(x)		( ( x & REQID_BUS_8_MASK ) 		<< REQID_BUS_8_OFST )

#define BUS_MASTER_ENABLE_MASK	( 0x04 )

#define SOP_WORD	( 0x01 )
#define EOP_WORD	( 0x02 )

//
// Global variables
//
alt_u32 reqid_value;

int receive_1DW_completion( alt_u32* received_tag, alt_u32* received_data ) {

	alt_u32 header_0;
	alt_u32 header_1;
	alt_u32 header_2;
	alt_u32 data;
	alt_u32 pad_0;
	alt_u32 pad_1;
	alt_u32 temp;
	alt_u32 unaligned = 0;
	alt_u32 alignment_error = 0;

	// wait for some rx data to arrive
	while( IORD_ALTERA_AVALON_FIFO_LEVEL( PCIEB_RAW_TLP_RX_FIFO_IN_CSR_BASE ) == 0 );

	header_0 = IORD_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_RX_FIFO_OUT_BASE );
	// make sure that we've just grabbed the first word of a packet, otherwise we're out of sync with the data stream
	if( ( IORD_ALTERA_AVALON_FIFO_OTHER_INFO( PCIEB_RAW_TLP_RX_FIFO_OUT_BASE ) & SOP_WORD ) != SOP_WORD ) {
		return( -1 );
	}

	// now I'm going to assume that the rest of the packet is sitting in the fifo, which is probably not a safe thing to do

	header_1 = IORD_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_RX_FIFO_OUT_BASE );
	header_2 = IORD_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_RX_FIFO_OUT_BASE );

	// see if we're receiving an unaligned word
	if( ( header_2 & 0x00000004 ) != 0 ) {

		unaligned = 1;

	} else {

		unaligned = 0;
	}

	temp = IORD_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_RX_FIFO_OUT_BASE );
	// now we could get EOP on this word or in three words from now
	if( ( IORD_ALTERA_AVALON_FIFO_OTHER_INFO( PCIEB_RAW_TLP_RX_FIFO_OUT_BASE ) & EOP_WORD ) != EOP_WORD ) {

		pad_0 = temp;
		if( unaligned == 1 )
			alignment_error = 1;

		temp = IORD_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_RX_FIFO_OUT_BASE );
		// we would NOT expect EOP here
		if( ( IORD_ALTERA_AVALON_FIFO_OTHER_INFO( PCIEB_RAW_TLP_RX_FIFO_OUT_BASE ) & EOP_WORD ) != EOP_WORD ) {

			data = temp;

			temp = IORD_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_RX_FIFO_OUT_BASE );
			// we would expect EOP here
			if( ( IORD_ALTERA_AVALON_FIFO_OTHER_INFO( PCIEB_RAW_TLP_RX_FIFO_OUT_BASE ) & EOP_WORD ) != EOP_WORD ) {

				// well at this point lets try flushing the fifo until we get to the EOP
				IORD_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_RX_FIFO_OUT_BASE );
				while( ( IORD_ALTERA_AVALON_FIFO_OTHER_INFO( PCIEB_RAW_TLP_RX_FIFO_OUT_BASE ) & EOP_WORD ) != EOP_WORD ) {

					IORD_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_RX_FIFO_OUT_BASE );
				}
				return( -6 );

			} else {

				pad_1 = temp;
			}
		} else {

			return( -5 );
		}
	} else {

		data = temp;
		if( unaligned == 0 )
			alignment_error = 1;
	}

	// check for alignment errors
	if( alignment_error != 0 ) {

		return( - 7 );
	}

	// check the completion status bits
	if( ( header_1 & 0x0000E000 ) != 0 ) {
		return( -2 );
	}

	// check the length
	if( ( header_0 & 0x000003FF ) != 1 ) {
		return( -3 );
	}

	// check the byte count
	if( ( header_1 & 0x00000FFF ) != 4 ) {
		return( -4 );
	}

	*received_tag = ( header_2 >> 8 ) & 0xFF;
	*received_data = data;
	return(0);
}

void send_3DW_write1DW_tlp( alt_u32 address, alt_u32 data ) {

	alt_u32 next_dw;

	// create header word 0
	next_dw = 	HDR_LENGTH_10(1)	|
				HDR_RSV0_2(0)		|
				HDR_ATTR_2(0)		|
				HDR_EP_1(0)			|
				HDR_TD_1(0)			|
				HDR_RSV1_4(0)		|
				HDR_TC_3(0)			|
				HDR_RSV2_1(0)		|
				HDR_TYPE_5(0)		|
				HDR_FMT_2(2)		|
				HDR_RSV3_1(0);

	IOWR_ALTERA_AVALON_FIFO_OTHER_INFO( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, SOP_WORD );	// set the SOP bit in the fifo
	IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, next_dw );			// write header 0 to the fifo

	// create header word 1
	next_dw = 	HDR_FDWBE_4( 0xF )			|
				HDR_LDWBE_4( 0 )			|
				HDR_TAG_8( 0 )			|
				HDR_REQID_16( reqid_value );

	IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, next_dw );			// write header 1 to the fifo

	// create header word 2
	next_dw = 	address & 0xFFFFFFFC;

	IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, next_dw );			// write header 2 to the fifo

	if( ( address & 0x04 ) != 0 ) {		// memory transaction is NOT QW aligned

		IOWR_ALTERA_AVALON_FIFO_OTHER_INFO( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, EOP_WORD );	// set the EOP bit in the fifo
		IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, data );			// write data word to the fifo

	} else {							// memory transaction IS QW aligned

		IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, 0x00 );			// write padding word to the fifo
		IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, data );			// write data word to the fifo
		IOWR_ALTERA_AVALON_FIFO_OTHER_INFO( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, EOP_WORD );	// set the EOP bit in the fifo
		IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, 0x00 );			// write padding word to the fifo
	}
}

void send_4DW_write1DW_tlp( alt_u32 address_hi, alt_u32 address_lo, alt_u32 data ) {

	alt_u32 next_dw;

	// create header word 0
	next_dw = 	HDR_LENGTH_10(1)	|
				HDR_RSV0_2(0)		|
				HDR_ATTR_2(0)		|
				HDR_EP_1(0)			|
				HDR_TD_1(0)			|
				HDR_RSV1_4(0)		|
				HDR_TC_3(0)			|
				HDR_RSV2_1(0)		|
				HDR_TYPE_5(0)		|
				HDR_FMT_2(3)		|
				HDR_RSV3_1(0);

	IOWR_ALTERA_AVALON_FIFO_OTHER_INFO( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, SOP_WORD );	// set the SOP bit in the fifo
	IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, next_dw );			// write header 0 to the fifo

	// create header word 1
	next_dw = 	HDR_FDWBE_4( 0xF )			|
				HDR_LDWBE_4( 0 )			|
				HDR_TAG_8( 0 )			|
				HDR_REQID_16( reqid_value );

	IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, next_dw );			// write header 1 to the fifo

	// create header word 2
	next_dw = 	address_hi;

	IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, next_dw );			// write header 2 to the fifo

	// create header word 3
	next_dw = 	address_lo & 0xFFFFFFFC;

	IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, next_dw );			// write header 3 to the fifo

	if( ( address_lo & 0x04 ) != 0 ) {	// memory transaction is NOT QW aligned

		IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, 0x00 );			// write padding word to the fifo
		IOWR_ALTERA_AVALON_FIFO_OTHER_INFO( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, EOP_WORD );	// set the EOP bit in the fifo
		IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, data );			// write data word to the fifo

	} else {							// memory transaction IS QW aligned

		IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, data );			// write data word to the fifo
		IOWR_ALTERA_AVALON_FIFO_OTHER_INFO( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, EOP_WORD );	// set the EOP bit in the fifo
		IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, 0x00 );			// write padding word to the fifo
	}
}

void send_3DW_read1DW_tlp( alt_u32 address, alt_u32 tag ) {

	alt_u32 next_dw;

	// create header word 0
	next_dw = 	HDR_LENGTH_10(1)	|
				HDR_RSV0_2(0)		|
				HDR_ATTR_2(0)		|
				HDR_EP_1(0)			|
				HDR_TD_1(0)			|
				HDR_RSV1_4(0)		|
				HDR_TC_3(0)			|
				HDR_RSV2_1(0)		|
				HDR_TYPE_5(0)		|
				HDR_FMT_2(0)		|
				HDR_RSV3_1(0);

	IOWR_ALTERA_AVALON_FIFO_OTHER_INFO( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, SOP_WORD );	// set the SOP bit in the fifo
	IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, next_dw );			// write header 0 to the fifo

	// create header word 1
	next_dw = 	HDR_FDWBE_4( 0xF )			|
				HDR_LDWBE_4( 0 )			|
				HDR_TAG_8( tag )			|
				HDR_REQID_16( reqid_value );

	IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, next_dw );			// write header 1 to the fifo

	// create header word 2
	next_dw = 	address & 0xFFFFFFFC;

	IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, next_dw );			// write header 2 to the fifo

	IOWR_ALTERA_AVALON_FIFO_OTHER_INFO( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, EOP_WORD );	// set the EOP bit in the fifo
	IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, 0x00 );			// write padding word to the fifo
}

void send_4DW_read1DW_tlp( alt_u32 address_hi, alt_u32 address_lo, alt_u32 tag ) {

	alt_u32 next_dw;

	// create header word 0
	next_dw = 	HDR_LENGTH_10(1)	|
				HDR_RSV0_2(0)		|
				HDR_ATTR_2(0)		|
				HDR_EP_1(0)			|
				HDR_TD_1(0)			|
				HDR_RSV1_4(0)		|
				HDR_TC_3(0)			|
				HDR_RSV2_1(0)		|
				HDR_TYPE_5(0)		|
				HDR_FMT_2(1)		|
				HDR_RSV3_1(0);

	IOWR_ALTERA_AVALON_FIFO_OTHER_INFO( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, SOP_WORD );	// set the SOP bit in the fifo
	IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, next_dw );			// write header 0 to the fifo

	// create header word 1
	next_dw = 	HDR_FDWBE_4( 0xF )			|
				HDR_LDWBE_4( 0 )			|
				HDR_TAG_8( tag )			|
				HDR_REQID_16( reqid_value );

	IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, next_dw );			// write header 1 to the fifo

	// create header word 2
	next_dw = 	address_hi;

	IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, next_dw );			// write header 2 to the fifo

	// create header word 3
	next_dw = 	address_lo & 0xFFFFFFFC;

	IOWR_ALTERA_AVALON_FIFO_OTHER_INFO( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, EOP_WORD );	// set the EOP bit in the fifo
	IOWR_ALTERA_AVALON_FIFO_DATA( PCIEB_RAW_TLP_TX_FIFO_IN_BASE, next_dw );			// write header 3 to the fifo
}

int main()
{
	alt_u32 lmi_reg;
	volatile int i;
	alt_u32 busdev_value;
	alt_u32 bus_number;
	alt_u32 dev_number;
	alt_u32 func_number;
	int j;
	int result;
	alt_u32 rec_tags[8];
	alt_u32 rec_data[8];

		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, ENABLE_MODULE_PRINT, 0 );
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, ENABLE_PRINT_STORE, 0 );

		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_CHAR_RAW, ( '1' << 24 )|( 'v' << 16 )|( 'e' << 8 )|( 'R' << 0 ));
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_LINE_FEED, 0x00);
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, DUMP_PRINT_STORE, 0 );

		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_LINE_FEED, 0x00);
		simPrintString("Hello from Nios II!");
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_LINE_FEED, 0x00);
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, DUMP_PRINT_STORE, 0 );

	lmi_reg = IORD_32DIRECT( PCIEB_LMI_SLAVE_BASE, 0x00 );

		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_LINE_FEED, 0x00);
		simPrintString("Dev ID and Vendor ID = ");
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_HEX_TYPED, lmi_reg);
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_LINE_FEED, 0x00);
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, DUMP_PRINT_STORE, 0 );

	lmi_reg = IORD_32DIRECT( PCIEB_LMI_SLAVE_BASE, 0x04 );

		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_LINE_FEED, 0x00);
		simPrintString("Status and Command = ");
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_HEX_TYPED, lmi_reg);
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_LINE_FEED, 0x00);
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, DUMP_PRINT_STORE, 0 );

		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_LINE_FEED, 0x00);
		simPrintString("Waiting for Bus Master Enable.");
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_LINE_FEED, 0x00);
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, DUMP_PRINT_STORE, 0 );

	// wait for the RP to set Bus Master Enable as our signal to go.
	lmi_reg = IORD_32DIRECT( PCIEB_LMI_SLAVE_BASE, 0x04 );
	while( (lmi_reg & BUS_MASTER_ENABLE_MASK) == 0 ) {

		// I have noticed that if you continually access the LMI interface back to back as fast a Nios can
		// in sumulation the interface tends to become non-responsive.  I'm not sure if it's completely dead
		// or if it's just incurred an extremely long access latency.  Anyway, if I induce some latency between
		// Nios II accesses to the LMI interface, like 1us or so, then this behavior does not seem to appear.
		// Even with some artificial latency like this, you can still see access times on the order of 10us or so
		// depending on the configuration activity occuring over the PCIe interface.
		for( i = 0 ; i < 10 ; i++ );

		lmi_reg = IORD_32DIRECT( PCIEB_LMI_SLAVE_BASE, 0x04 );
	}

		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_LINE_FEED, 0x00);
		simPrintString("Saw Bus Master Enable as ");
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_HEX_TYPED, lmi_reg);
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_LINE_FEED, 0x00);
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, DUMP_PRINT_STORE, 0 );

	// get our bus and device number so we can create raw TLP packets
	busdev_value = IORD_32DIRECT( PCIEB_BUSDEV_SLAVE_BASE, 0x00 );
	bus_number = ( busdev_value >> 5 ) & 0xFF;
	dev_number = ( busdev_value >> 0 ) & 0x1F;
	func_number = 0;

	reqid_value = REQID_BUS_8(bus_number) | REQID_DEV_5(dev_number) | REQID_FUNC_3(func_number);

		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_CHAR_RAW, ( '=' << 24 )|( 'F' << 16 )|( 'D' << 8 )|( 'B' << 0 ));
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_HEX_TYPED, reqid_value );
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_LINE_FEED, 0x00);
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, DUMP_PRINT_STORE, 0 );

	// wait for the RP to set MSI message data to 0xCAFE as our signal to go.
	lmi_reg = IORD_32DIRECT( PCIEB_LMI_SLAVE_BASE, 0x5C );
	while( ( lmi_reg & 0xFFFF ) != 0xCAFE ) {

		for( i = 0 ; i < 10 ; i++ );

		lmi_reg = IORD_32DIRECT( PCIEB_LMI_SLAVE_BASE, 0x5C );
	}

		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_LINE_FEED, 0x00);
		simPrintString("Saw MSI message data as ");
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_HEX_TYPED, lmi_reg);
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_LINE_FEED, 0x00);
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, DUMP_PRINT_STORE, 0 );

	// memory span on RP that has been set to FF is 0x08000 thru 0x18000 other simulation tests use the range
	// of 0x08000 and 0x10000 for a span of nearly 0x08000 so I'm going to use the last 512 bytes of the span
	// and hope that I don't collide with the other simulation tests.

	// first in 3DW space below 4GB

	// write some DWs
	send_3DW_write1DW_tlp( 0x17FFC, 0x88888888 );
	send_3DW_write1DW_tlp( 0x17FF8, 0x77777777 );
	send_3DW_write1DW_tlp( 0x17FF4, 0x66666666 );
	send_3DW_write1DW_tlp( 0x17FF0, 0x55555555 );
	send_3DW_write1DW_tlp( 0x17FEC, 0x44444444 );
	send_3DW_write1DW_tlp( 0x17FE8, 0x33333333 );
	send_3DW_write1DW_tlp( 0x17FE4, 0x22222222 );
	send_3DW_write1DW_tlp( 0x17FE0, 0x11111111 );

	// read them back
	send_3DW_read1DW_tlp( 0x17FFC, 16 );
	send_3DW_read1DW_tlp( 0x17FF8, 17 );
	send_3DW_read1DW_tlp( 0x17FF4, 18 );
	send_3DW_read1DW_tlp( 0x17FF0, 19 );
	send_3DW_read1DW_tlp( 0x17FEC, 20 );
	send_3DW_read1DW_tlp( 0x17FE8, 21 );
	send_3DW_read1DW_tlp( 0x17FE4, 22 );
	send_3DW_read1DW_tlp( 0x17FE0, 23 );

	for( j = 0 ; j < 8 ; j++ ) {

		result = receive_1DW_completion( &rec_tags[j], &rec_data[j] );
		if( result < 0 ) {

				simPrintString("Error during receive completions on pass ");
				IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_DEC_TYPED, j);
				IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_HEX_TYPED, result);
				IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_DEC_TYPED, result);
				IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_DEC_TYPED, -result);
				IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_LINE_FEED, 0x00);
				IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, DUMP_PRINT_STORE, 0 );

				IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, STOP_FAIL, 0x00);

			break;
		}
	}

	if(
		( rec_tags[0] != 16 ) ||
		( rec_tags[1] != 17 ) ||
		( rec_tags[2] != 18 ) ||
		( rec_tags[3] != 19 ) ||
		( rec_tags[4] != 20 ) ||
		( rec_tags[5] != 21 ) ||
		( rec_tags[6] != 22 ) ||
		( rec_tags[7] != 23 ) ||
		( rec_data[0] != 0x88888888 ) ||
		( rec_data[1] != 0x77777777 ) ||
		( rec_data[2] != 0x66666666 ) ||
		( rec_data[3] != 0x55555555 ) ||
		( rec_data[4] != 0x44444444 ) ||
		( rec_data[5] != 0x33333333 ) ||
		( rec_data[6] != 0x22222222 ) ||
		( rec_data[7] != 0x11111111 )
	) {

		simPrintString("Data or Tag validation error...");
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_LINE_FEED, 0x00);
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, DUMP_PRINT_STORE, 0 );

		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, STOP_FAIL, 0x00);
	}

	// next in 4DW space at 4GB

	// write some DWs
	send_4DW_write1DW_tlp( 0x00000001, 0x17FDC, 0x80808080 );
	send_4DW_write1DW_tlp( 0x00000001, 0x17FD8, 0x70707070 );
	send_4DW_write1DW_tlp( 0x00000001, 0x17FD4, 0x60606060 );
	send_4DW_write1DW_tlp( 0x00000001, 0x17FD0, 0x50505050 );
	send_4DW_write1DW_tlp( 0x00000001, 0x17FCC, 0x40404040 );
	send_4DW_write1DW_tlp( 0x00000001, 0x17FC8, 0x30303030 );
	send_4DW_write1DW_tlp( 0x00000001, 0x17FC4, 0x20202020 );
	send_4DW_write1DW_tlp( 0x00000001, 0x17FC0, 0x10101010 );

	// read them back
	send_4DW_read1DW_tlp( 0x00000001, 0x17FDC, 16 );
	send_4DW_read1DW_tlp( 0x00000001, 0x17FD8, 17 );
	send_4DW_read1DW_tlp( 0x00000001, 0x17FD4, 18 );
	send_4DW_read1DW_tlp( 0x00000001, 0x17FD0, 19 );
	send_4DW_read1DW_tlp( 0x00000001, 0x17FCC, 20 );
	send_4DW_read1DW_tlp( 0x00000001, 0x17FC8, 21 );
	send_4DW_read1DW_tlp( 0x00000001, 0x17FC4, 22 );
	send_4DW_read1DW_tlp( 0x00000001, 0x17FC0, 23 );

	for( j = 0 ; j < 8 ; j++ ) {

		result = receive_1DW_completion( &rec_tags[j], &rec_data[j] );
		if( result < 0 ) {

				simPrintString("Error during receive completions on pass ");
				IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_DEC_TYPED, j);
				IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_HEX_TYPED, result);
				IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_DEC_TYPED, result);
				IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_DEC_TYPED, -result);
				IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_LINE_FEED, 0x00);
				IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, DUMP_PRINT_STORE, 0 );

				IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, STOP_FAIL, 0x00);

			break;
		}
	}

	if(
		( rec_tags[0] != 16 ) ||
		( rec_tags[1] != 17 ) ||
		( rec_tags[2] != 18 ) ||
		( rec_tags[3] != 19 ) ||
		( rec_tags[4] != 20 ) ||
		( rec_tags[5] != 21 ) ||
		( rec_tags[6] != 22 ) ||
		( rec_tags[7] != 23 ) ||
		( rec_data[0] != 0x80808080 ) ||
		( rec_data[1] != 0x70707070 ) ||
		( rec_data[2] != 0x60606060 ) ||
		( rec_data[3] != 0x50505050 ) ||
		( rec_data[4] != 0x40404040 ) ||
		( rec_data[5] != 0x30303030 ) ||
		( rec_data[6] != 0x20202020 ) ||
		( rec_data[7] != 0x10101010 )
	) {

		simPrintString("Data or Tag validation error...");
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, PRINT_LINE_FEED, 0x00);
		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, DUMP_PRINT_STORE, 0 );

		IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, STOP_FAIL, 0x00);
	}

	IOWR_32DIRECT( SIMULATIONPRINT_0_BASE, STOP_SUCCESS, 0x00);

	/* Event loop never exits. */
	while (1);

	return 0;
}
