module register_wrapper
#(
	parameter INSIG_WIDTH  = 0,
	parameter OUTSIG_WIDTH = 0
)
(
	clk,
	reset_n,

	insig_in,
	outsig_out
);

localparam SLAVE_ADDRESS_WIDTH = 28;

input	wire							clk;
input	wire							reset_n;

input	[  ( INSIG_WIDTH - 1 ) : 0 ]	insig_in;
output	[ ( OUTSIG_WIDTH - 1 ) : 0 ]	outsig_out;

reg		[  ( INSIG_WIDTH - 1 ) : 0 ]	reg_in;
reg		[ ( OUTSIG_WIDTH - 1 ) : 0 ]	reg_out;

wire	[  ( INSIG_WIDTH - 1 ) : 0 ]	in_vect;
wire	[ ( OUTSIG_WIDTH - 1 ) : 0 ]	out_vect;

reg										my_reset_n;

//
// instance input signal wires
//
wire  clk_sig;
wire  reset_sig;
wire [15:0] asi_devcsrsnk_data_sig;
wire [12:0] asi_busdevsnk_data_sig;
wire [SLAVE_ADDRESS_WIDTH-1:0] avs_s0_address_sig;
wire [7:0] avs_s0_byteenable_sig;
wire [9:0] avs_s0_burstcount_sig;
wire  avs_s0_read_sig;
wire  avs_s0_write_sig;
wire [63:0] avs_s0_writedata_sig;
wire [1:0] avs_csr_address_sig;
wire [3:0] avs_csr_byteenable_sig;
wire  avs_csr_read_sig;
wire  avs_csr_write_sig;
wire [31:0] avs_csr_writedata_sig;
wire  avm_tagreq_waitrequest_sig;
wire [63:0] asi_rddatasnk_data_sig;
wire  asi_rddatasnk_valid_sig;
wire  asi_rddatasnk_startofpacket_sig;
wire  asi_rddatasnk_endofpacket_sig;
wire  aso_eoplbesrc_ready_sig;
wire  aso_tlpsrc_ready_sig;

//
// instance output signal wires
//
wire  avs_s0_waitrequest_sig;
wire [63:0] avs_s0_readdata_sig;
wire  avs_s0_readdatavalid_sig;
wire [31:0] avs_csr_readdata_sig;
wire [7:0] avm_tagreq_address_sig;
wire [3:0] avm_tagreq_byteenable_sig;
wire  avm_tagreq_write_sig;
wire [31:0] avm_tagreq_writedata_sig;
wire  asi_rddatasnk_ready_sig;
wire [4:0] aso_eoplbesrc_data_sig;
wire  aso_eoplbesrc_valid_sig;
wire [63:0] aso_tlpsrc_data_sig;
wire  aso_tlpsrc_valid_sig;
wire  aso_tlpsrc_startofpacket_sig;
wire  aso_tlpsrc_endofpacket_sig;

//
// assign clock and reset drivers
//
assign clk_sig = clk;
//assign FIXME_reset_n_sig = my_reset_n;	//  my_reset_n for negative asserted reset
assign reset_sig = ~my_reset_n;		// ~my_reset_n for positive asserted reset

//
// map input and output registers to input and output vectors
//
assign in_vect = reg_in;
assign outsig_out = reg_out;

//
// map input and output vectors to instance inputs and outputs
//
assign {
	asi_devcsrsnk_data_sig,
	asi_busdevsnk_data_sig,
	avs_s0_address_sig,
	avs_s0_byteenable_sig,
	avs_s0_burstcount_sig,
	avs_s0_read_sig,
	avs_s0_write_sig,
	avs_s0_writedata_sig,
	avs_csr_address_sig,
	avs_csr_byteenable_sig,
	avs_csr_read_sig,
	avs_csr_write_sig,
	avs_csr_writedata_sig,
	avm_tagreq_waitrequest_sig,
	asi_rddatasnk_data_sig,
	asi_rddatasnk_valid_sig,
	asi_rddatasnk_startofpacket_sig,
	asi_rddatasnk_endofpacket_sig,
	aso_eoplbesrc_ready_sig,
	aso_tlpsrc_ready_sig
} = in_vect;

assign out_vect = {
	avs_s0_waitrequest_sig,
	avs_s0_readdata_sig,
	avs_s0_readdatavalid_sig,
	avs_csr_readdata_sig,
	avm_tagreq_address_sig,
	avm_tagreq_byteenable_sig,
	avm_tagreq_write_sig,
	avm_tagreq_writedata_sig,
	asi_rddatasnk_ready_sig,
	aso_eoplbesrc_data_sig,
	aso_eoplbesrc_valid_sig,
	aso_tlpsrc_data_sig,
	aso_tlpsrc_valid_sig,
	aso_tlpsrc_startofpacket_sig,
	aso_tlpsrc_endofpacket_sig
};

always @ ( posedge clk ) begin
	
		my_reset_n <= reset_n;
end

always @ ( posedge clk or negedge my_reset_n ) begin
	
	if( ~my_reset_n ) begin
	
		reg_in <= 0;
		reg_out <= 0;
		
	end else begin
	
		reg_in <= insig_in;
		reg_out <= out_vect;

	end
end

//
// instanctiate test instance here
//
slave2tlp_64b slave2tlp_64b_inst
(
	.clk(clk_sig) ,	// input  clk_sig
	.reset(reset_sig) ,	// input  reset_sig
	.asi_devcsrsnk_data(asi_devcsrsnk_data_sig) ,	// input [15:0] asi_devcsrsnk_data_sig
	.asi_busdevsnk_data(asi_busdevsnk_data_sig) ,	// input [12:0] asi_busdevsnk_data_sig
	.avs_s0_address(avs_s0_address_sig) ,	// input [SLAVE_ADDRESS_WIDTH-1:0] avs_s0_address_sig
	.avs_s0_byteenable(avs_s0_byteenable_sig) ,	// input [7:0] avs_s0_byteenable_sig
	.avs_s0_burstcount(avs_s0_burstcount_sig) ,	// input [9:0] avs_s0_burstcount_sig
	.avs_s0_read(avs_s0_read_sig) ,	// input  avs_s0_read_sig
	.avs_s0_write(avs_s0_write_sig) ,	// input  avs_s0_write_sig
	.avs_s0_writedata(avs_s0_writedata_sig) ,	// input [63:0] avs_s0_writedata_sig
	.avs_s0_waitrequest(avs_s0_waitrequest_sig) ,	// output  avs_s0_waitrequest_sig
	.avs_s0_readdata(avs_s0_readdata_sig) ,	// output [63:0] avs_s0_readdata_sig
	.avs_s0_readdatavalid(avs_s0_readdatavalid_sig) ,	// output  avs_s0_readdatavalid_sig
	.avs_csr_address(avs_csr_address_sig) ,	// input [1:0] avs_csr_address_sig
	.avs_csr_byteenable(avs_csr_byteenable_sig) ,	// input [3:0] avs_csr_byteenable_sig
	.avs_csr_read(avs_csr_read_sig) ,	// input  avs_csr_read_sig
	.avs_csr_write(avs_csr_write_sig) ,	// input  avs_csr_write_sig
	.avs_csr_writedata(avs_csr_writedata_sig) ,	// input [31:0] avs_csr_writedata_sig
	.avs_csr_readdata(avs_csr_readdata_sig) ,	// output [31:0] avs_csr_readdata_sig
	.avm_tagreq_address(avm_tagreq_address_sig) ,	// output [7:0] avm_tagreq_address_sig
	.avm_tagreq_byteenable(avm_tagreq_byteenable_sig) ,	// output [3:0] avm_tagreq_byteenable_sig
	.avm_tagreq_write(avm_tagreq_write_sig) ,	// output  avm_tagreq_write_sig
	.avm_tagreq_writedata(avm_tagreq_writedata_sig) ,	// output [31:0] avm_tagreq_writedata_sig
	.avm_tagreq_waitrequest(avm_tagreq_waitrequest_sig) ,	// input  avm_tagreq_waitrequest_sig
	.asi_rddatasnk_data(asi_rddatasnk_data_sig) ,	// input [63:0] asi_rddatasnk_data_sig
	.asi_rddatasnk_ready(asi_rddatasnk_ready_sig) ,	// output  asi_rddatasnk_ready_sig
	.asi_rddatasnk_valid(asi_rddatasnk_valid_sig) ,	// input  asi_rddatasnk_valid_sig
	.asi_rddatasnk_startofpacket(asi_rddatasnk_startofpacket_sig) ,	// input  asi_rddatasnk_startofpacket_sig
	.asi_rddatasnk_endofpacket(asi_rddatasnk_endofpacket_sig) ,	// input  asi_rddatasnk_endofpacket_sig
	.aso_eoplbesrc_data(aso_eoplbesrc_data_sig) ,	// output [4:0] aso_eoplbesrc_data_sig
	.aso_eoplbesrc_ready(aso_eoplbesrc_ready_sig) ,	// input  aso_eoplbesrc_ready_sig
	.aso_eoplbesrc_valid(aso_eoplbesrc_valid_sig) ,	// output  aso_eoplbesrc_valid_sig
	.aso_tlpsrc_data(aso_tlpsrc_data_sig) ,	// output [63:0] aso_tlpsrc_data_sig
	.aso_tlpsrc_ready(aso_tlpsrc_ready_sig) ,	// input  aso_tlpsrc_ready_sig
	.aso_tlpsrc_valid(aso_tlpsrc_valid_sig) ,	// output  aso_tlpsrc_valid_sig
	.aso_tlpsrc_startofpacket(aso_tlpsrc_startofpacket_sig) ,	// output  aso_tlpsrc_startofpacket_sig
	.aso_tlpsrc_endofpacket(aso_tlpsrc_endofpacket_sig) 	// output  aso_tlpsrc_endofpacket_sig
);

defparam slave2tlp_64b_inst.SLAVE_ADDRESS_WIDTH = SLAVE_ADDRESS_WIDTH;
defparam slave2tlp_64b_inst.MAX_READ_REQ_SIZE = 5;
defparam slave2tlp_64b_inst.AVAILABLE_TAG_COUNT = 16;
defparam slave2tlp_64b_inst.BASE_TAG_VALUE = 0;

endmodule
