//----------------------------------------------------------------------------------------------------
//-- (c)2010 Altera Corporation. All rights reserved.
//--
//-- Altera products are protected under numerous U.S. and foreign patents,
//-- maskwork rights, copyrights and other intellectual property laws.
//--
//-- This reference design file, and your use thereof, is subject to and governed
//-- by the terms and conditions of the applicable Altera Reference Design License
//-- Agreement (either as signed by you or found at www.altera.com).  By using
//-- this reference design file, you indicate your acceptance of such terms and
//-- conditions between you and Altera Corporation.  In the event that you do not
//-- agree with such terms and conditions, you may not use the reference design
//-- file and please promptly destroy any copies you have made.
//--
//-- This reference design file is being provided on an as-is basis and as an
//-- accommodation and therefore all warranties, representations or guarantees of
//-- any kind (whether express, implied or statutory) including, without
//-- limitation, warranties of merchantability, non-infringement, or fitness for
//-- a particular purpose, are specifically disclaimed.  By making this reference
//-- design file available, Altera expressly does not recommend, suggest or
//-- require that this reference design file be used in combination with any
//-- other product not provided by Altera.
//----------------------------------------------------------------------------------------------------

//
//	This module receives 64-bit descriptors which classifies into read and write descriptors and assigns the appropriate
//  Avalon ST channel indication.
//
//	  input descriptor 64-bit: { desc_type[ 0 : 0 ], reserved[ 14 : 0 ], channel[ 7 : 0 ], word_count[ 7 : 0 ], address[ 31 : 0 ] }
//
//	write descriptor : desc_type == 1'b0 : channel = 0
//	 read descriptor : desc_type == 1'b1 : channel = 1
//	
	
module dma_descriptor_classify_64b (

	// Clock Interface
	input  wire				clk,
	input  wire				reset,
	
	// Descriptor Input Interface - Avalon ST
	input  wire	[ 63 : 0 ]	asi_snk_data,
	output wire				asi_snk_ready,
	input  wire				asi_snk_valid,
	
	// Classified Output Interface - Avalon ST
	output reg	[ 63 : 0 ]	aso_src_data,
	input  wire				aso_src_ready,
	output wire				aso_src_valid,
	output reg				aso_src_channel
);

//
// output Pipeline
//
dma_descriptor_classify_64b_1stage_pipeline #( .PAYLOAD_WIDTH( 65 ) ) outpipe (
    .clk            (clk),
    .reset_n        (~reset),
    .in_ready       (asi_snk_ready),
    .in_valid       (asi_snk_valid), 
    .in_payload     ( { asi_snk_data[63], asi_snk_data } ),
    .out_ready      (aso_src_ready), 
    .out_valid      (aso_src_valid), 
    .out_payload    ( { aso_src_channel, aso_src_data } )
);

endmodule

//  --------------------------------------------------------------------------------
// | single buffered pipeline stage
//  --------------------------------------------------------------------------------
module dma_descriptor_classify_64b_1stage_pipeline  
#( parameter PAYLOAD_WIDTH = 8 )
 ( input                          clk,
   input                          reset_n, 
   output reg                     in_ready,
   input                          in_valid,   
   input      [PAYLOAD_WIDTH-1:0] in_payload,
   input                          out_ready,   
   output reg                     out_valid,
   output reg [PAYLOAD_WIDTH-1:0] out_payload      
 );
      
   always @* begin
     in_ready <= out_ready || ~out_valid;
   end
   
   always @ (negedge reset_n, posedge clk) begin
      if (!reset_n) begin
         out_valid <= 0;
         out_payload <= 0;
      end else begin
         if (in_valid) begin
           out_valid <= 1;
         end else if (out_ready) begin
           out_valid <= 0;
         end
         
         if(in_valid && in_ready) begin
            out_payload <= in_payload;
         end
      end
   end

endmodule
