//----------------------------------------------------------------------------------------------------
//-- (c)2010 Altera Corporation. All rights reserved.
//--
//-- Altera products are protected under numerous U.S. and foreign patents,
//-- maskwork rights, copyrights and other intellectual property laws.
//--
//-- This reference design file, and your use thereof, is subject to and governed
//-- by the terms and conditions of the applicable Altera Reference Design License
//-- Agreement (either as signed by you or found at www.altera.com).  By using
//-- this reference design file, you indicate your acceptance of such terms and
//-- conditions between you and Altera Corporation.  In the event that you do not
//-- agree with such terms and conditions, you may not use the reference design
//-- file and please promptly destroy any copies you have made.
//--
//-- This reference design file is being provided on an as-is basis and as an
//-- accommodation and therefore all warranties, representations or guarantees of
//-- any kind (whether express, implied or statutory) including, without
//-- limitation, warranties of merchantability, non-infringement, or fitness for
//-- a particular purpose, are specifically disclaimed.  By making this reference
//-- design file available, Altera expressly does not recommend, suggest or
//-- require that this reference design file be used in combination with any
//-- other product not provided by Altera.
//----------------------------------------------------------------------------------------------------

//	
//	This module receives 64-bit descriptors thru it's Avalon Slave interface and it translates them into an Avalon ST stream.
//

module read_master_descriptor_slave_64b
(
	// Clock Interface
	input  wire				clk,
	input  wire				reset,
	
	// Descriptor Output Interface - Avalon ST
	output wire	[ 63 : 0 ]	aso_descriptor_data,
	input  wire				aso_descriptor_ready,
	output wire				aso_descriptor_valid,

	// Slave Interface - Avalon MM
	input  wire	[  8 : 0 ]	avs_s0_address,
	input  wire	[  7 : 0 ]	avs_s0_byteenable,
	input  wire				avs_s0_write,
	output wire				avs_s0_waitrequest,
	input  wire	[ 63 : 0 ]	avs_s0_writedata
);

wire			descriptor_in_ready;
wire			in_valid;

assign in_valid = descriptor_in_ready & avs_s0_write & ( avs_s0_byteenable == 8'hFF );

assign avs_s0_waitrequest = ~in_valid;

//
// Descriptor output Pipeline
//
read_master_descriptor_slave_64b_1stage_pipeline #( .PAYLOAD_WIDTH( 64 ) ) descriptor_outpipe (
    .clk            ( clk ),
    .reset_n        ( ~reset ),
    .in_ready       ( descriptor_in_ready ),
    .in_valid       ( in_valid ), 
    .in_payload     ( avs_s0_writedata ),
    .out_ready      ( aso_descriptor_ready ), 
    .out_valid      ( aso_descriptor_valid ), 
    .out_payload    ( aso_descriptor_data )
);

endmodule

//  --------------------------------------------------------------------------------
// | single buffered pipeline stage
//  --------------------------------------------------------------------------------
module read_master_descriptor_slave_64b_1stage_pipeline  
#( parameter PAYLOAD_WIDTH = 8 )
 ( input                          clk,
   input                          reset_n, 
   output reg                     in_ready,
   input                          in_valid,   
   input      [PAYLOAD_WIDTH-1:0] in_payload,
   input                          out_ready,   
   output reg                     out_valid,
   output reg [PAYLOAD_WIDTH-1:0] out_payload      
 );
      
   always @* begin
     in_ready <= out_ready || ~out_valid;
   end
   
   always @ (negedge reset_n, posedge clk) begin
      if (!reset_n) begin
         out_valid <= 0;
         out_payload <= 0;
      end else begin
         if (in_valid) begin
           out_valid <= 1;
         end else if (out_ready) begin
           out_valid <= 0;
         end
         
         if(in_valid && in_ready) begin
            out_payload <= in_payload;
         end
      end
   end

endmodule
