//----------------------------------------------------------------------------------------------------
//-- (c)2010 Altera Corporation. All rights reserved.
//--
//-- Altera products are protected under numerous U.S. and foreign patents,
//-- maskwork rights, copyrights and other intellectual property laws.
//--
//-- This reference design file, and your use thereof, is subject to and governed
//-- by the terms and conditions of the applicable Altera Reference Design License
//-- Agreement (either as signed by you or found at www.altera.com).  By using
//-- this reference design file, you indicate your acceptance of such terms and
//-- conditions between you and Altera Corporation.  In the event that you do not
//-- agree with such terms and conditions, you may not use the reference design
//-- file and please promptly destroy any copies you have made.
//--
//-- This reference design file is being provided on an as-is basis and as an
//-- accommodation and therefore all warranties, representations or guarantees of
//-- any kind (whether express, implied or statutory) including, without
//-- limitation, warranties of merchantability, non-infringement, or fitness for
//-- a particular purpose, are specifically disclaimed.  By making this reference
//-- design file available, Altera expressly does not recommend, suggest or
//-- require that this reference design file be used in combination with any
//-- other product not provided by Altera.
//----------------------------------------------------------------------------------------------------

//	
//	This module receives Avalon master commands in a descriptor with the following format:
//
//	address descriptor 40-bit: { word_count[ 7 : 0 ], address[ 31 : 0 ] }
//
//	It uses the address descriptor to generate write requests thru it's Avalon Master interface.  The write
//	data is provided thru the Write Data Avalon ST interface.
//	

module write_master_64b
(
	// Clock Interface
	input  wire				clk,
	input  wire				reset,
	
	// Avalon Address Descriptor Input Interface - Avalon ST
	input  wire	[ 39 : 0 ]	asi_addrdesc_data,
	output wire				asi_addrdesc_ready,
	input  wire				asi_addrdesc_valid,

	// Avalon Master Write Data Input Interface - Avalon ST
	input  wire	[ 63 : 0 ]	asi_wrdatasnk_data,
	output wire				asi_wrdatasnk_ready,
	input  wire				asi_wrdatasnk_valid,

	// Master Interface - Avalon MM
	output wire	[ 31 : 0 ]	avm_m0_address,
	output wire	[  7 : 0 ]	avm_m0_byteenable,
	output wire	[  9 : 0 ]	avm_m0_burstcount,
	output wire				avm_m0_write,
	input  wire				avm_m0_waitrequest,
	output wire	[ 63 : 0 ]	avm_m0_writedata
);

//
// Misc Variables
//
reg  [ 7 : 0 ] word_sent_count;
wire [ 7 : 0 ] next_word_sent_count;
wire		   last_word_in_transfer;

//
// Master Command Input Elements
//
wire [ 31 : 0 ] avm_address;
wire [  7 : 0 ]	avm_burstcount;

assign { avm_burstcount, avm_address } = asi_addrdesc_data;

assign avm_m0_address = avm_address;
assign avm_m0_byteenable = 8'hFF;
assign avm_m0_burstcount = avm_burstcount;
assign avm_m0_writedata = asi_wrdatasnk_data;

assign avm_m0_write = asi_addrdesc_valid & asi_wrdatasnk_valid;

assign asi_addrdesc_ready = last_word_in_transfer & avm_m0_write & ~avm_m0_waitrequest;

assign asi_wrdatasnk_ready = avm_m0_write & ~avm_m0_waitrequest;

assign next_word_sent_count = word_sent_count + 1;

assign last_word_in_transfer = ( next_word_sent_count == avm_burstcount ) ? ( 1'b1 ) : ( 1'b0 );

always @ ( posedge clk or posedge reset ) begin
	if( reset ) begin
	
		word_sent_count <= 0;
		
	end else begin
		if( asi_wrdatasnk_ready ) begin
			if( last_word_in_transfer ) begin
			
				word_sent_count <= 0;
				
			end else begin

				word_sent_count <= next_word_sent_count;
			end
		end
	end
end

endmodule
