//----------------------------------------------------------------------------------------------------
//-- (c)2010 Altera Corporation. All rights reserved.
//--
//-- Altera products are protected under numerous U.S. and foreign patents,
//-- maskwork rights, copyrights and other intellectual property laws.
//--
//-- This reference design file, and your use thereof, is subject to and governed
//-- by the terms and conditions of the applicable Altera Reference Design License
//-- Agreement (either as signed by you or found at www.altera.com).  By using
//-- this reference design file, you indicate your acceptance of such terms and
//-- conditions between you and Altera Corporation.  In the event that you do not
//-- agree with such terms and conditions, you may not use the reference design
//-- file and please promptly destroy any copies you have made.
//--
//-- This reference design file is being provided on an as-is basis and as an
//-- accommodation and therefore all warranties, representations or guarantees of
//-- any kind (whether express, implied or statutory) including, without
//-- limitation, warranties of merchantability, non-infringement, or fitness for
//-- a particular purpose, are specifically disclaimed.  By making this reference
//-- design file available, Altera expressly does not recommend, suggest or
//-- require that this reference design file be used in combination with any
//-- other product not provided by Altera.
//----------------------------------------------------------------------------------------------------

//	
//	This module receives 64-bit descriptors which it splits into an Avalon ST stream, an address descriptor stream.
//
//	  input descriptor 64-bit: { reserved[ 15 : 0 ], channel[ 7 : 0 ], word_count[ 7 : 0 ], address[ 31 : 0 ] }
//
//	address descriptor 40-bit: { word_count[ 7 : 0 ], address[ 31 : 0 ] }
//

module write_master_descriptor_separator_64b
(
	// Clock Interface
	input  wire				clk,
	input  wire				reset,
	
	// Address Descriptor Output Interface - Avalon ST
	output wire	[ 39 : 0 ]	aso_addrdesc_data,
	input  wire				aso_addrdesc_ready,
	output wire				aso_addrdesc_valid,

	// Input Descriptor Interface - Avalon ST
	input  wire	[ 63 : 0 ]	asi_descriptor_data,
	output wire				asi_descriptor_ready,
	input  wire				asi_descriptor_valid
);

//
// Descriptor Elements
//
wire [ 15 : 0 ] reserved;
wire [  7 : 0 ]	channel;
wire [  7 : 0 ]	word_count;
wire [ 31 : 0 ] address;

wire			addr_desc_in_ready;
wire			in_valid;

assign { reserved[ 15 : 0 ], channel[ 7 : 0 ], word_count[ 7 : 0 ], address[ 31 : 0 ] } = asi_descriptor_data;

assign asi_descriptor_ready = addr_desc_in_ready;

assign in_valid = asi_descriptor_ready & asi_descriptor_valid;

//
// Address Descriptor output Pipeline
//
write_master_descriptor_separator_64b_1stage_pipeline #( .PAYLOAD_WIDTH( 40 ) ) addr_desc_outpipe (
    .clk            ( clk ),
    .reset_n        ( ~reset ),
    .in_ready       ( addr_desc_in_ready ),
    .in_valid       ( in_valid ), 
    .in_payload     ( { word_count[ 7 : 0 ], address[ 31 : 0 ] } ),
    .out_ready      ( aso_addrdesc_ready ), 
    .out_valid      ( aso_addrdesc_valid ), 
    .out_payload    ( aso_addrdesc_data )
);

endmodule

//  --------------------------------------------------------------------------------
// | single buffered pipeline stage
//  --------------------------------------------------------------------------------
module write_master_descriptor_separator_64b_1stage_pipeline  
#( parameter PAYLOAD_WIDTH = 8 )
 ( input                          clk,
   input                          reset_n, 
   output reg                     in_ready,
   input                          in_valid,   
   input      [PAYLOAD_WIDTH-1:0] in_payload,
   input                          out_ready,   
   output reg                     out_valid,
   output reg [PAYLOAD_WIDTH-1:0] out_payload      
 );
      
   always @* begin
     in_ready <= out_ready || ~out_valid;
   end
   
   always @ (negedge reset_n, posedge clk) begin
      if (!reset_n) begin
         out_valid <= 0;
         out_payload <= 0;
      end else begin
         if (in_valid) begin
           out_valid <= 1;
         end else if (out_ready) begin
           out_valid <= 0;
         end
         
         if(in_valid && in_ready) begin
            out_payload <= in_payload;
         end
      end
   end

endmodule
